function parseUICommand(parameters) {
	if (parameters.action === undefined) {
		// do nothing if action is undefined
		return;
	} else {
		var PropTreeReady = WescoVars.get("Controller/PropTreeReady");

		if (PropTreeReady !== null && PropTreeReady !== undefined) {

			switch (parameters.action) {
				case "RediscoverDevices_Airboxes":
					// https://192.168.3.159/json/event/raise?name=wesco_UI&parameter=action=RediscoverAirboxes
					airboxes.discover();
					break;

				case "RediscoverDevices_Exhoods":
					// https://192.168.3.159/json/event/raise?name=wesco_UI&parameter=action=RediscoverAirboxes
					exhood.discover();
					break;

				case "RediscoverDevices_WCFans":
					// https://192.168.3.159/json/event/raise?name=wesco_UI&parameter=action=RediscoverAirboxes
					wcfans.discover();
					break;

				case "SetWCFanRegs":
					var dsuid = parameters.dsuid;
					var dict = parameters.Regs;
					wcfans.setRegs(dsuid, dict);
					break;

				case "RefreshPushTargetOptions":
					// create a list of connected mobile devices
					try {
						cloudService.fetchNotificationList(notifications.createList);

					} catch (err) {
						// report error,  not cloud activated
					}
					break;

				case "ClearAllAirboxErrors":
					//airboxes.clearError("302ed89f43f0000000000d0000099cf800");
					//airboxes.setToLastCalledScene();
					//openExhaustFlap();
					break;

				case "ClearAirboxError":
					var dsuid = parameters.dSUID;
					airboxes.clearError(dsuid);
					break;


				case "StoredConfigurableParameters":
					WescoVars.store();
					debug("INFO_2", "Stored complete property tree");
					break;

				case "StoredExhaustFlap":
					WescoVars.store();
					debug("INFO_2", "Stored complete property tree");
					openExhaustFlap();

					break;

				case "SetAirboxesLevel":

					var Dict = {};
					var level = parseInt(parameters.level); // 0,1,2,3,4 -> [0,17,18,19,5];
					var parameter = {};
					parameter["level"] = level;
					var eventUI = new Event("wesco_UIVentilationLevelEvent", parameter);
					var eventID = eventUI.raise();


					// var scenen_levels=[0,5,17,18,19]; //
					// var scene=scenen_levels[level];
					// var args=[scene];
					// var param={};

					// WescoVars.save("Controller/CurrentLevel",level,false);
					// WescoVars.save("Controller/CurrentScene",scene,true);

					// param['path_toDevices']='Airboxes/Devices/';
					// param['action']='sendScene';
					// param['args']=JSON.stringify(args);
					// param['FSMState']="initialize";
					// param['Dict']=JSON.stringify(Dict);
					// tiny.SendMultiple(param);

					break;

				case "configVentilationSchedule":
					//https://192.168.5.100/json/event/raise?name=wesco_UI&parameter=action=configVentilationSchedule;schedule={%221%22:{%228%22:{%22presence%22:3,%22absence%22:2},%229%22:{%22presence%22:2,%22absence%22:2},%2217%22:{%22presence%22:3,%22absence%22:2},%2218%22:{%22presence%22:2,%22absence%22:2}},%222%22:{%228.00%22:{%22presence%22:3,%22absence%22:2},%229.00%22:{%22presence%22:2,%22absence%22:2},%2217.00%22:{%22presence%22:3,%22absence%22:2},%2218%22:{%22presence%22:2,%22absence%22:2}}}

					debug("INFO_2", "Config:" + JSON.stringify(parameters));
					var schedule = parameters.schedule; // stringified dictionary {"day":{"hh":0|1,"hh2":0|1 ...},"day2":{...} //0=off,1=on

					WescoVars.configureVentilationSchedule(schedule);
					break;

				case "TinyRegs":
					//https://192.168.5.100/json/event/raise?name=wesco_UI&parameter=action=AirboxRegs;subaction=readSingle;dSID=302ed89f43f00d0000099b30;args=[0,12,13,14,15]
					//https://192.168.5.100/json/event/raise?name=wesco_UI&parameter=action=AirboxRegs;subaction=readAll;args=[0,12,13,14,15]
					//https://192.168.5.100/json/event/raise?name=wesco_UI&parameter=action=AirboxRegs;subaction=writeAll;args=[[12,200],[13,101],[14,120],[15,139],[3,4]]. Writing

					//https://192.168.5.100/json/event/raise?name=wesco_UI&parameter=action=AirboxRegs;subaction=writeSingle;dSID=302ed89f43f00d0000099b30;args=[[12,200],[13,102],[14,121],[15,140],[3,4]]. Writing
					//register 3 to value 4 stores the values to flash.
					var Dict = {};
					var subaction = parameters.subaction;
					var args = parameters.args;
					var type = parameters.type;


					var param = {};
					if (type == 'Airbox') {
						param['path_toDevices'] = 'Airboxes/Devices/';
					} else {
						param['path_toDevices'] = 'ExHood/Devices/';
					}
					param['args'] = args;
					param['FSMState'] = "initialize";
					param['Dict'] = JSON.stringify(Dict);
					param['action'] = subaction; // writeSingle or
					// writeAll. If
					// writeSingle, dSID
					// must be specified

					if (subaction == 'writeSingle' || subaction == 'readSingle') {
						if (parameters.dSID !== undefined) {
							param['dSID'] = parameters.dSID;
						} else {
							break; // skip the operation, dSID missing
						}
					}


					var busy = WescoVars.get_create("Controller/writingInProgress", false, false);
					debug("INFO_2", "parameters:" + JSON.stringify(parameters) + ";busy:" + busy);
					if (busy == false) {
						tiny.SendMultiple(param);
					} else if (busy == true) {
						//var timer=timedevents.timers["TIMER_defer_DS_Action"];
						//timedevents.call(timer.name,timer.id,"+120",param);
					}
					break;

				default:
					break;
			} // end 	switch(parameters.action){
		} //end if
	}
}