var Airboxes = function() {


	this.discover = function() {
		/*
		  This function scans all devices in zone0 at the property tree
		  and retrieves those with EAN number matching WESCO_AIRBOX_EAN.

		  The associated airbox variables at the property tree are
		  retrieved/initialized.
		*/



		function removeNonPresentDevices() {
			/*This function recursevely removes from the property tree
			 * devices that are not anymore present.  This function
			 * does not yet consider subfolders (only one level per
			 * airbox).
			 */

			var airboxesPath = "Airboxes/Devices";

			try {
				var prop = Property.getNode(airboxesPath);
				var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist


				for (var i in array) { // iterate over the airboxes nodes
					var AirboxName = array[i].getName();
					var AirboxPath = array[i].getPath();

					var isValid = ConfigurationIsValid(AirboxPath);

					if (!isValid) { // if device is not present
						var subprop = Property.getNode(AirboxPath);
						var subarray = subprop.getChildren();

						// iterate over all airbox properties and delete them
						for (var j in subarray) {
							var item = subarray[j].getName();
							subprop.removeChild(item);
						}
						// once all the variables were removed, remove the
						// airbox node.
						prop.removeChild(AirboxName);

						debug("INFO_1", "Removed airbox device:" + AirboxID);
					} else {
						try {
							var AirboxID = WescoVars.get(AirboxPath + "/dSID");
							var currentAirbox = getDevices().byDSID(AirboxID);

							if (!currentAirbox.isPresent) { // if device is not present
								var subprop = Property.getNode(AirboxPath);
								var subarray = subprop.getChildren();

								// iterate over all airbox properties and delete them
								for (var j in subarray) {
									var item = subarray[j].getName();
									subprop.removeChild(item);
								}
								// once all the variables were removed, remove the
								// airbox node.
								prop.removeChild(AirboxName);

								debug("INFO_1", "Removed airbox device:" + AirboxID);
								WESCOLog("W", ErrorCodes["deletedAirbox_from_propTree"], AirboxID, "");
							}
						} catch (err) {
							debug("ERROR", "Exception while removing airbox:" + err);

							var subprop = Property.getNode(AirboxPath);
							var subarray = subprop.getChildren();

							// iterate over all airbox properties and delete them
							for (var j in subarray) {
								var item = subarray[j].getName();
								subprop.removeChild(item);
							}
							// once all the variables were removed, remove the
							// airbox node.
							prop.removeChild(AirboxName);
							debug("INFO_1", "Removed airbox  device:" + AirboxID);
							WESCOLog("W", ErrorCodes["deletedAirbox_from_propTree"], AirboxID, "");
						}
					}
				} //end for
			} //end try
			catch (err) {
				debug("ERROR", "Exception while removing airbox " + err);
				//do nothing. For instance if the directory Devices does not yet exist
			}
		} // end ...     function removeNonPresentDevices(){



		function createDefaultAirbox(dsuid, id) {
			// Initializes/retrieves variables corresponding to each
			// airbox.

			var path = "Airboxes/Devices/" + dsuid + "/"
			WescoVars.save(path + "dSID", id, false);
			WescoVars.get_create(path + "errorCode", ErrorCodes["no_error"], false);
			WescoVars.get_create(path + "Status", 0x00, false);
			WescoVars.get_create(path + "errorTime", DEFAULT_STRING, false);
			WescoVars.get_create(path + "errorCounter", 0, false);
			WescoVars.get_create(path + "errorFSMState", "ready", false);
			WescoVars.get_create(path + "SoftwareVersion", 0x32, false);
		} // end ...    function createDefaultAirbox(dsuid,id){

		removeNonPresentDevices(); // remove entries of non present devices


		// the list of airboxes is created based on the devices in zone0,
		// whose EAN matches the Airbox one.
		var Nairboxes = 0;

		getDevices().perform(function(device) {

			var path = "/apartment/zones/zone0/devices/" + device.dsuid + "/"
			var EAN = WescoVars.get(path + "productInfo/EAN");
			if (EAN == WESCO_AIRBOX_EAN) {
				Nairboxes = Nairboxes + 1;
			}

			if (device.isPresent) {
				var dSID = WescoVars.get(path + "dSID");

				// if the EAN number matches the WESCO one, then create an airbox node.
				if (EAN == WESCO_AIRBOX_EAN) {
					debug("INFO_2", "Create airbox node;EAN:" + EAN + ";dSID:" + dSID + ";dsuid:" + device.dsuid + ";Name:" + device.name);

					// check if the airbox is already at the property tree
					var AirboxPath = "Airboxes/Devices/" + device.dsuid;
					var prop = Property.getNode(AirboxPath);

					if (prop === undefined || prop === null) {
						WESCOLog("I", ErrorCodes["registeredAirbox_to_propTree"], dSID, "");
					}
					createDefaultAirbox(device.dsuid, dSID);
				}
			}
		});


		// makes the uninstallable flag conditional. The app can be
		// uninstall only if all airbox devices are inactive and have
		// been removed.
		if (Nairboxes > 0) {
			WescoVars.save("system/preventUninstall", true, false);
			debug("INFO_2", "making up uninstallable");
		} else {
			WescoVars.save("system/preventUninstall", false, false);
			debug("INFO_2", "making up non-uninstallable");
		}

		WescoVars.store();
		debug("INFO_2", "Completed Airbox discovery");
	};


	function ConfigurationIsValid(path) {
		// Returns true if these fields exist
		try {
			if (WescoVars.get(path + "/dSID") == undefined) {
				return false;
			}
			if (WescoVars.get(path + "/errorCode") == undefined) {
				return false;
			}
			if (WescoVars.get(path + "/Status") == undefined) {
				return false;
			}
			if (WescoVars.get(path + "/errorTime") == undefined) {
				return false;
			}
			if (WescoVars.get(path + "/errorCounter") == undefined) {
				return false;
			}
			if (WescoVars.get(path + "/errorFSMState") == undefined) {
				return false;
			}
			return true;
		} catch (err) {
			debug("WARN_1", "Configuration Invalid: " + path);
			return false;
		}
	}; // end 	function ConfigurationIsValid(path){



	this.getActiveDevices = function() {
		var airboxesPath = "Airboxes/Devices";
		var NAirboxes = 0;

		try {
			var prop = Property.getNode(airboxesPath);
			var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

			for (var i in array) { // iterate over the airboxes nodes
				var AirboxName = array[i].getName();
				var AirboxPath = array[i].getPath();

				if (ConfigurationIsValid(AirboxPath)) { // if configuration is valid
					var AirboxID = WescoVars.get(AirboxPath + "/dSID");
					var currentAirbox = getDevices().byDSID(AirboxID);
					if (currentAirbox.isPresent) { // if device is not present
						var status = parseInt(WescoVars.get(AirboxPath + "/Status"));
						if (status == 0) { // if this airbox is not in error 0x40 or locked 0x01
							NAirboxes++;
						}
					}
				} else {
					debug("ERROR", "Error while getting number of active airboxes; dsuid:" + AirboxID);
				}
			}
			return NAirboxes;
		} catch (err) {
			debug("ERROR", "Exception while getting number of active airboxes:" + err);
			return NAirboxes;
		} //end try
	}; // end ...     function removeNonPresentDevices(){



	this.isAirbox = function(dsuid) {
		/*
		  This function verifies wheather a dsuid belongs to an airbox or not.
		  The function does not verify if the device is present or not.
		*/

		var AirboxPath = "Airboxes/Devices/" + dsuid;
		var prop = Property.getNode(AirboxPath);

		if (prop) {
			return true;
		} else {
			return false;
		}
	};

	this.setStatus = function(dsuid, mask) {
		// This warning means that the airbox is attempting a frost
		// error removal. During this time is the airbox locked or not
		// available for ventilation.
		var AirboxPath = "Airboxes/Devices/" + dsuid;
		var prev_status = parseInt(WescoVars.get(AirboxPath + "/Status"));
		WescoVars.save(AirboxPath + "/Status", mask | prev_status, false);
	};

	this.clearStatus = function(dsuid, mask) {
		// This warning means that the airbox is attempting a frost
		// error removal. During this time is the airbox locked or not
		// available for ventilation.
		var AirboxPath = "Airboxes/Devices/" + dsuid;
		var prev_status = parseInt(WescoVars.get(AirboxPath + "/Status"));
		WescoVars.save(AirboxPath + "/Status", prev_status & ~mask, false);
	};


	this.removeError = function(dsuid, ErrorCode, Caller) {
		/*
		  This FSM implements the error handling for a particular airbox defined by its dsuid.

		  The second parameter is the ErrorCode as received from the
		  airbox Status Event.  The ErrorCode is only relevant at the ready state.

		  Caller identifies the source  that has invoked the FSM.  Caller can be "errorEvent" or "Timer".

		  The FSM has two states:

		  ready: if an error arrives, checks based on the date of the
		  previous occurrence wheather it is the first occurrence or not.
		  Some errors are reported immediately.

		  The state machine attempts to remove some errors. If the number
		  of tries is larger than AIRBOX_CLEARERROR_NTRIES -1, then the error is
		  reported.

		  The error removal consist of writing airbox' register 3 to 2

		  This FSM is only called if the variable
		  Airboxes/Devices/dsuid/errorCode is undefined or is equal to
		  NOERROR.  This variable has to be reset (to NOERROR) from the
		  UI (see function clearAllAirboxErrors).

		*/


		function nextFSMState(AirboxPath, state) {
			// writes the next state
			WescoVars.save(AirboxPath + "/errorFSMState", state, true);
		}

		function logError(ErrorCode, AirboxID, msg) {

			//Writes the corresponding error code to the log file.
			//Returns the code.

			//Error decoding: status bit structure:

			// [7] Fehler
			// [6] Serviceanforderung (nicht definiert)
			// [5] Frostwarnung
			// [4] Frostgefahr / Frostfehler
			// [3] Klappenfehler
			// [2] Ventilatorfehler
			// [1] Sensorfehler
			// [0] Kein Fehler

			var error = parseInt(ErrorCode);
			var errorString = ErrorCodes["no_error"];

			if (error == 128) {
				errorString = ErrorCodes["power_error"];
				WESCOLog("E", errorString, AirboxID, msg);
			}
			if (error & (1 << 1)) {
				errorString = ErrorCodes["sensor_error"];
				WESCOLog("E", errorString, AirboxID, msg);
			}
			if (error & (1 << 2)) {
				errorString = ErrorCodes["fan_error"];
				WESCOLog("E", errorString, AirboxID, msg);
			}
			if (error & (1 << 3)) {
				errorString = ErrorCodes["flap_error"];
				WESCOLog("E", errorString, AirboxID, msg);
			}

			if (error & (1 << 5)) { // deprecated
				errorString = ErrorCodes["frost_warning"];
				WESCOLog("W", errorString, AirboxID, msg);
			}

			if (error & (1 << 4)) {
				errorString = ErrorCodes["frost_error"];
				WESCOLog("E", errorString, AirboxID, msg);
			}

			return errorString;
		} //end ... function logError(ErrorCode,AirboxID,msg){


		var ErrorStr;
		var AirboxPath = "Airboxes/Devices/" + dsuid;
		var dsid = WescoVars.get(AirboxPath + "/dSID");
		var dnow = new Date(); // current date

		// arms the corresponding timer for the different FSM transitions.
		var timer = timedevents.timers["TIMER_clearupAirboxError"];
		timer.id = timer.id + dsuid; //create a per-device timer
		var parameter = {
			"dsuid": dsuid
		}; // this dictionary is passed to the timer call function



		// gets the current FSM state from the property tree [ready,clearing_on,clearing_off]
		var errorFSMState = WescoVars.get_create(AirboxPath + "/errorFSMState", "ready", true);


		switch (errorFSMState) {
			case "ready":
				debug("INFO_3", "case ready;dsuid:" + dsuid);
				var doSoftwareErrorRemoval = true;

				try {
					var version = parseInt(WescoVars.get(AirboxPath + "/SoftwareVersion"));
					debug("INFO_3", "version = " + version);
					if (version >= 0x32) { // if version is 3.2 or newer
						// Versions > 3.2 include the error removal in the
						// airbox itself and it is therefore not necessary
						// to do the error removal by software

						doSoftwareErrorRemoval = false;

						debug("INFO_3", "reporting error");
						ErrorStr = logError(ErrorCode, dsid, "Error removal in the airbox failed");
						//writes the error code  to the property tree
						WescoVars.save(AirboxPath + "/errorCode", ErrorStr, true);

						nextFSMState(AirboxPath, "ready");

						debug("INFO_2", "sending notification");
						//------------------------------------------
						// Notification
						notifications.schedule("Error");
						debug("INFO_2", "sending email");
						notifications.sendEmailError(ErrorStr, dsid);
						//------------------------------------------

						WescoVars.save(AirboxPath + "/errorTime", dnow.toISOString(), false);
						WescoVars.store();
					}
				} catch (err) {
					debug("WARN_1", "Exception while getting Airbox version");
					doSoftwareErrorRemoval = truel;
				}


				if (doSoftwareErrorRemoval) {
					switch (parseInt(ErrorCode)) {

						case AIRBOX_SENSOR_ERR:
						case AIRBOX_FLAP_ERR: // sensor error  and flap error

							// No error removal attempt for those errors.  They are reported as soon as they appear.

							ErrorStr = logError(ErrorCode, dsid, "No error removal attempt");
							WescoVars.save(AirboxPath + "/errorCode", ErrorStr, false);
							WescoVars.save(AirboxPath + "/errorTime", dnow.toISOString(), false);
							WescoVars.store();
							closeExhaustFlap();

							debug("INFO_2", "sending notification");
							//------------------------------------------
							// Notification
							notifications.schedule("Error");
							debug("INFO_2", "sending email");
							notifications.sendEmailError(ErrorStr, dsid);

							break;

						case AIRBOX_POWER_ERR:
						case AIRBOX_FAN_ERR:
						case AIRBOX_FROST_ERR: // power error, fan error  and sensor error

							// for those errors the error removal function is carried out

							// time of previous error occurrence
							var dprev = WescoVars.get_create(AirboxPath + "/errorTime", dnow.toISOString(), true);

							if (dprev === undefined || dprev == DEFAULT_STRING) {
								dprev = new Date(0);
							} else {
								dprev = new Date(dprev);
							}

							// number of times the error has occurred
							var errorCounter = parseInt(WescoVars.get(AirboxPath + "/errorCounter"));
							if (errorCounter === undefined || errorCounter == DEFAULT_INT) {
								errorCounter = 0;
							}

							//elapsed time in seconds between the present and the previous error occurrences
							var DeltaTime = parseInt(dnow - dprev) / 1000; // in  seconds


							// if the elapsed time < AIRBOX_MAX_TIME_BTW_ERRORS increase the errorCounter
							if (DeltaTime < AIRBOX_MAX_TIME_BTW_ERRORS) {
								if (errorCounter == 0) {
									logError(ErrorCode, dsid, "First occurrence");
								}
								errorCounter++;

							} else {
								// if the elapsed time is larger or equal to
								// AIRBOX_MAX_TIME_BTW_ERRORS then reset the counter
								// and increment. Log the first error occurrence
								logError(ErrorCode, dsid, "First occurrence");
								errorCounter = 1; // restart to zero and increase
							}

							closeExhaustFlap();

							// The error removal is triggered only a limited number of
							// times. After that, no more attempts are carried out and
							// the error is reported. Timer durations are defined in timer.js

							if (errorCounter < AIRBOX_CLEARERROR_NTRIES) {
								if (parseInt(ErrorCode) == AIRBOX_POWER_ERR) { // power_error
									timedevents.call(timer.name, timer.id, POWER_ERROR_CLEAR_WAIT, parameter);
								} else if (parseInt(ErrorCode) == AIRBOX_FAN_ERR) {
									timedevents.call(timer.name, timer.id, FAN_ERROR_CLEAR_WAIT, parameter);
								} else {
									timedevents.call(timer.name, timer.id, FROST_ERROR_CLEAR_WAIT, parameter);
								}

								// restart the ntries counter, how many times one has
								// tried to write the error removal command
								WescoVars.save(AirboxPath + "/clearErrorNtries", 0, true);

								// next FSM state: goes to set the clearing error function
								nextFSMState(AirboxPath, "clearing_on");

							} else {
								// if the error could not be removed ...
								//log the error
								debug("INFO_3", "reporting error");
								ErrorStr = logError(ErrorCode, dsid, "Error removal failed");
								//writes the error code  to the property tree
								WescoVars.save(AirboxPath + "/errorCode", ErrorStr, true);

								nextFSMState(AirboxPath, "ready");

								debug("INFO_2", "sending notification");
								//------------------------------------------
								// Notification
								notifications.schedule("Error");
								debug("INFO_2", "sending email");
								notifications.sendEmailError(ErrorStr, dsid);
								//------------------------------------------

							}
							WescoVars.save(AirboxPath + "/errorTime", dnow.toISOString(), false);
							WescoVars.save(AirboxPath + "/errorCounter", errorCounter, false);
							WescoVars.store();
							break;
						default:
							break;
					} // end ..	switch(parseInt(ErrorCode)){
				} // if doSoftwareErrorRemoval


				break; //end ... case "ready":

			case "clearing_on":

				if (Caller == "errorEvent") {
					// When a new error arrives and calls the FSM with
					// Caller=errorEvent, the FSM processes the error only if
					// it is at the ready state. Otherwise, it means that the
					// FSM is busy.
					break;
				}

				debug("INFO_3", "clearing_on;dsuid:" + dsuid);
				// writes the airbox's special command register to remove
				// the error.  After a few seconds this register has to be
				// written back  to 0, this is done in the clearing_off state.
				// Writing the register back to 0 is not necessary in the airbox firmaware version 3.1
				try {
					tiny.writeReg(dsid, 3, 2); //write special register to 2 (clear up error)
					nextFSMState(AirboxPath, "restoreScene");
					timedevents.call(timer.name, timer.id, "+7", parameter);
				} catch (err) {
					var clearErrorNtries = parseInt(WescoVars.get_create(AirboxPath + "/clearErrorNtries", 0, true));
					clearErrorNtries++;

					if (clearErrorNtries > AIRBOX_WRITE_MAX_RETRY) {
						nextFSMState(AirboxPath, "ready");
						clearErrorNtries = 0;

						// notify error
						WESCOLog("E", ErrorCodes["timeout_error"], dsid, "");
						debug("WARN_1", "Airbox unreachable:" + dsuid);
					} else {
						nextFSMState(AirboxPath, "clearing_on");
						timedevents.call(timer.name, timer.id, "+60", parameter); //wait before the next attempt [seconds]
					}
					WescoVars.save(AirboxPath + "/clearErrorNtries", clearErrorNtries, true);
				}
				break;
			case "restoreScene":

				if (Caller == "errorEvent") {
					// When a new error arrives and calls the FSM with
					// Caller=errorEvent, the FSM processes the error only if
					// it is at the ready state. Otherwise, it means that the
					// FSM is busy.
					break;
				}

				debug("INFO_3", "restoreScene;dsuid:" + dsuid);
				try {
					this.callLastScene(dsid); //write special register to 2 (clear up error)
					nextFSMState(AirboxPath, "ready");
					timedevents.call(timer.name, timer.id, "+7", parameter);
				} catch (err) {
					var clearErrorNtries = parseInt(WescoVars.get_create(AirboxPath + "/clearErrorNtries", 0, true));
					clearErrorNtries++;

					if (clearErrorNtries > AIRBOX_WRITE_MAX_RETRY) {
						nextFSMState(AirboxPath, "ready");
						clearErrorNtries = 0;

						// notify error
						WESCOLog("E", ErrorCodes["timeout_error"], dsid, "");
						debug("WARN_1", "Airbox unreachable:" + dsuid);
					} else {
						nextFSMState(AirboxPath, "restoreScene");
						timedevents.call(timer.name, timer.id, "+60", parameter); //wait before the next attempt [seconds]
					}
					WescoVars.save(AirboxPath + "/clearErrorNtries", clearErrorNtries, true);
				}
				break;


			default:
				break;

		} //end ... switch(errorFSMState){
	}; // end ... function  AirboxErrorRoutine(dsuid,ErrorCode){



	this.clearError = function(dsuid) {
		var airboxesPath = "Airboxes/Devices";
		var AirboxPath = airboxesPath + "/" + dsuid;
		var AirboxID = WescoVars.get(AirboxPath + "/dSID");

		var error = tiny.readReg(AirboxID, 0); // read error register 0

		if (error & 0x01) { // if not error
			WescoVars.save(AirboxPath + "/errorCode", ErrorCodes["no_error"], false);
			//WescoVars.save(AirboxPath+"/Status",0x00,false);
			WescoVars.save(AirboxPath + "/errorCounter", 0, false);
			WescoVars.save(AirboxPath + "/errorFSMState", "ready", false);
			WescoVars.store();
			debug("INFO_2", "Reset error code of airbox " + AirboxID + ";error:" + error);

			// Send feedback to the UI

			return true;
		} else {
			debug("INFO_2", "Airbox still in error:" + AirboxID);

			// Send feedback to the UI

			return false;
		}

	}

	this.unlockAllAirboxes = function() {
		var airboxesPath = "Airboxes/Devices";
		var prop = Property.getNode(airboxesPath);
		var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist


		for (var i in array) {
			var AirboxPath = array[i].getPath();
			WescoVars.save(AirboxPath + "/errorFSMState", "ready", false);
			WescoVars.store();
			debug("INFO_2", "Reset errorFSM state for airbox " + AirboxPath);
		}
	}



	this.checkFilterReplacement = function() {
		/*This function returns true if the airbox filter has to be replaced*/

		var dateReplacement = WescoVars.get("Airboxes/LastFilterReplacementDate");
		var Today = new Date(); // current date

		if (dateReplacement !== undefined && dateReplacement != DEFAULT_STRING) {
			dateReplacement = new Date(dateReplacement);
			try {
				var Ndays = parseInt(WescoVars.get_create("Airboxes/FilterReplacementInterval", FilterReplacementInterval, false));
				var DeltaTime = parseInt(Today - dateReplacement) / (1000 * 86400); // in  days

				if (DeltaTime >= Ndays) {
					debug("INFO_2", "Ndays:" + Ndays + ";Deltatime:" + DeltaTime);
					WescoVars.save("Airboxes/LastFilterReplacementDate", Today.toISOString(), true); // reset date
					return true;
				} else {
					return false;
				}
			} catch (err) {
				// error in the conversion or format
				return false;
			}

		} else {
			return false;
		}

	};

	this.callLastScene = function(AirboxID) {
		/*This function restitutes the airboxes to the last called scene*/

		var lastScene = WescoVars.get("Controller/CurrentScene");

		if (lastScene !== null && lastScene !== undefined) {
			tiny.Scene(AirboxID, lastScene);
		}
	};

	this.updateVariables = function(Path, Reg, Value) {

		debug("INFO_3", "path:" + Path + ";Reg:" + Reg + ";Value:" + Value);
		if (Value === null || Path.indexOf("Airboxes") < 0) {
			return;
		}

		switch (Reg) {
			case 0: // error register
				var errorString = ErrorCodes["no_error"];
				if (Value == 128) {
					errorString = ErrorCodes["power_error"];
				}
				if (Value & (1 << 1)) {
					errorString = ErrorCodes["sensor_error"];
				}
				if (Value & (1 << 2)) {
					errorString = ErrorCodes["fan_error"];
				}
				if (Value & (1 << 3)) {
					errorString = ErrorCodes["flap_error"];
				}

				if (Value & (1 << 5)) {
					errorString = ErrorCodes["frost_warning"];
				}

				if (Value & (1 << 4)) {
					errorString = ErrorCodes["frost_error"]; // deprecated
				}
				WescoVars.save(Path + "/errorCode", errorString, false);

				break;

				//  % levels
			case 13:
			case 14:
			case 15:
				var levels = WescoVars.get("Controller/airbox_percentages");
				levels = JSON.parse(levels);
				levels[parseInt(Reg) - 13] = parseInt(Value);
				WescoVars.save("Controller/airbox_percentages", JSON.stringify(levels), false);
				break;

			case 62:
				debug("INFO_3","Got software version:"+Value);
				WescoVars.save(Path + "/SoftwareVersion", Value, false);
				break;

			default:
				break;
		} // end switch(Reg)
	}; // end function updateAirboxVariables(Path,Reg,Value){


}; // end object Airboxes


var airboxes = new Airboxes();



function closeExhaustFlap() {
	// closes the exhaust flap whenever one of the airboxes is in error
	var dSID = WescoVars.get("ExhaustFlap");

	if (dSID !== undefined && dSID != DEFAULT_STRING) {
		try {
			getDevices().byDSID(dSID).callScene(13); // dS minimum scene: 13
		} catch (err) {
			//report error. Device unreachable
		}
		debug("INFO_2", "closing flap:" + dSID);
	}
}

function openExhaustFlap() {

	// opens the exhaust flap whenever none of the airboxes are in error.

	// verifies that other airbox devices are not in error state before opening the exahust flap.
	var noerror = true;

	var airboxesPath = "Airboxes/Devices";
	var prop = Property.getNode(airboxesPath);
	var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

	for (var i in array) {
		var AirboxPath = array[i].getPath();
		var errorCode = WescoVars.get(AirboxPath + "/errorCode");
		var ID = WescoVars.get(AirboxPath + "/dSID");
		debug("INFO_2", "Airbox:" + ID + ";errorCode:" + errorCode);
		noerror = noerror && (errorCode === undefined || errorCode == ErrorCodes["no_error"]);
	}

	if (noerror) {
		open();
	}

	function open() {
		var dSID = WescoVars.get("ExhaustFlap");

		if (dSID !== undefined && dSID != DEFAULT_STRING) {
			try {
				getDevices().byDSID(dSID).callScene(14); // dS maximum scene: 14
			} catch (err) {
				//report error. Device unreachable
			}
			debug("INFO_2", "opening flap:" + dSID);
		}
	}
}