var Control = function() {

  this.processRules = function(type, value) {

    var AirboxLevelVolumes = JSON.parse(WescoVars.get("Controller/airbox_volume_levels"));
    var ExhoodLevelVolumes = JSON.parse(WescoVars.get("Controller/exhood_volume_levels"));

    function print_rules_gnuplot(Mode,
      PrevMode, isWindowOpen, NAirboxes, Vwc,
      airbox_level, flap_level, Vexhood, Vairbox) {

      /*This function is used for debugging purposes. It prints the current state 
      in tabulated format*/

      var flapValues = WescoVars.get("Controller/exhood_flap_percentages");

      flapValues = JSON.parse(flapValues);

      var flapPerc = flapValues[flap_level];
      var Vflap = flapPerc * Vexhood / 100;
      var imbalance = Vairbox + Vflap - Vexhood - Vwc;
      var to_print = "";
      //status
      to_print += Mode + ";";
      to_print += PrevMode + ";";
      if (isWindowOpen) {
        to_print += "1;";
      } else {
        to_print += "0;";
      }

      to_print += NAirboxes + ";";
      to_print += airbox_level + ";";
      to_print += flap_level + ";";
      to_print += Vairbox + ";";
      to_print += Vflap + ";";
      to_print += Vexhood + ";";
      to_print += Vwc + ";";
      to_print += imbalance + ";";

      to_print + "\n";

      debug("PLOT", to_print);
    }

    function get_airbox_level(volume) {
      /*Returns the corresponding level for a given volume.
        If the volume is larger than the stored levels, the maximum
        level is returned.
      */

      var levels = JSON.parse(WescoVars.get("Controller/airbox_volume_levels"));

      if (!isNaN(volume)) {
        for (var i in levels) {
          if (levels[i] >= volume / NAirboxes) {
            return i;
          }
        }
        return 4; // return maximum level
      } else {
        return 0;
      }
    }


    function get_exhood_volume(percent_value) {
      /*Calculates the volume corresponding to a given percentage. The
        volume depends on the calibration, it is, on the volume values configured
        for each level.*/
      try {
        var percentages = JSON.parse(WescoVars.get("Controller/exhood_fan_percentages"));
        var levels = JSON.parse(WescoVars.get("Controller/exhood_volume_levels"));

        //interpolate the volume
        for (var i = 0; i < Math.min(EXHOOD_NFANSCENES - 1, EXHOOD_NFLAPSCENES - 1); i++) {
          if (percent_value < percentages[i + 1]) {
            // do a linear interpolation
            var volume = levels[i + 1] - levels[i];
            volume = volume * percent_value / (percentages[i + 1] - percentages[i]);
            debug("INFO_1", "percent:" + percent_value + ";levels:" + levels[i] + "," + levels[i + 1] + ";volume:" + volume);
            return Math.abs(volume);
          }
        }
        var max_volume = levels[EXHOOD_NFANSCENES - 1];
        return percent_value * max_volume / 100;
      } catch (err) {
        debug("ERROR", "Exception in get_exhood_volume:" + err);
        return 715 * percent_value / 100;
      }
    }

    function get_flap_level(Vexhood, required_volume) {
      /* Calculates the flap level given the current ventilator volume
         and the total required volume. It is better to have excess than 
         default of ventilation; therefore, the flap is controlled towards 
         the excess side*/

      if (Vexhood > required_volume) {
        var percentage = required_volume * 100 / Vexhood;
        var flapValues = WescoVars.get("Controller/exhood_flap_percentages");

        flapValues = JSON.parse(flapValues);

        for (var i in flapValues) {
          if (percentage < flapValues[i]) {
            debug("INFO_1", "Returned Flap Level:" + flapValues[i] + "(" + i + ")");
            return parseInt(i);
          }
        }
      }
      debug("WARN_1", "Returning maximum flap opening");
      return EXHOOD_NFLAPSCENES; // there are 10 levels 0-9
    }

    function get_fan_level(required_volume) {
      /*
        Calculates the ventilator level given the required volume. 
        It is better to have excess than default of ventilation;
        therefore the fan is controlled towards the default side */

      var levels = JSON.parse(WescoVars.get("Controller/exhood_volume_levels"));

      debug("INFO_1", "Imbalance:" + required_volume);
      for (var i in levels) {
        if (required_volume <= levels[i] + 50) { // up to 50 m3/h positive balance is allowed
          debug("INFO_1", "level:" + i + "(" + levels[i] + ")");
          return i;
        }
      }
      return i;
    }

    function cooking_mode_rules(isWindowOpen, Vwc, Vexhood) {

      if (isWindowOpen) {

        /**************************************************************/
        /**************************************************************/
        /**************************************************************/
        // CONTROL RULE
        // Condition -> Cooking mode. Window is open, the window brings enough air to compensate the maximum exhaust possible.
        // Action -> if there is a value in the schedule (Zeitplan), set the airboxes to this level, otherwise set the airboxes to 0.

        // Base ventilation registers
        // Base ventilation registers
        var BaseVentilationLevel = 0;
        var BaseVentilationActive = WescoVars.get("Controller/BaseVentilation/Enabled");
        if (BaseVentilationActive == true) {
          BaseVentilationLevel = parseInt(WescoVars.get("Controller/BaseVentilation/Level"));
          if (isNaN(BaseVentilationLevel) || (BaseVentilationLevel < 0 || BaseVentilationLevel > 4)) {
            BaseVentilationLevel = 0;
          }
        }

        var airbox_level = BaseVentilationLevel;
        var Vairbox = NAirboxes * AirboxLevelVolumes[airbox_level];
        var airbox_scene = AirboxScenes[airbox_level];
        print_rules_gnuplot(Mode, PrevMode, isWindowOpen, NAirboxes, Vwc, airbox_level, 0, Vexhood, Vairbox);

      } else {
        /**************************************************************/
        /**************************************************************/
        /**************************************************************/
        // CONTROL RULE
        // Condition -> Cooking mode. Window is closed. The airboxes air volume is larger than the air exhaust.
        // Action -> the function finds out which airbox level has the minimum error. Flap is turned off.

        var airbox_level = 4; // start with level 4 -> FULL
        var Vairbox = NAirboxes * AirboxLevelVolumes[airbox_level];

        if (Vairbox > Vexhood + Vwc) {
          // find which levels is the most appropriate -> optimization (MIP)
          var better_balance = Math.abs(Vairbox - Vexhood - Vwc);
          var airbox_levels = [3, 2, 1, 0];
          for (var l = 0; l < airbox_levels.length; l++) {
            var level = airbox_levels[l];
            Vairbox = NAirboxes * AirboxLevelVolumes[level];
            var balance = Math.abs(Vairbox - Vexhood - Vwc);
            if (balance < better_balance) {
              better_balance = balance;
              airbox_level = airbox_levels[l];
            }
          }

          tiny.Scene(ExHood_dSID, ExHoodFlapScenes[0]); //turn the flap off, we don't need it anymore

          WescoVars.save("Controller/ExhoodCurrentValue_flap", 0, true);

          var scene = AirboxScenes[airbox_level];
          sendAirboxSceneDeferred(scene, "+5"); // send scene to all airboxes in 5 seconds time.

          print_rules_gnuplot(Mode, PrevMode, isWindowOpen, NAirboxes, Vwc, airbox_level, 0, Vexhood, Vairbox);

        }
        /**************************************************************/
        /**************************************************************/
        /**************************************************************/
        // CONTROL RULE
        // Condition -> Cooking mode. Window is closed. The airboxes air volume is smaller than the air exhaust.
        // Action -> Airboxes are set to full, the flap is controlled to compensate the imbalance.
        else {
          var airbox_level = 4;
          var Vairbox = NAirboxes * AirboxLevelVolumes[airbox_level];

          var airbox_scene = AirboxScenes[airbox_level];
          sendAirboxSceneDeferred(airbox_scene, "+5");

          var imbalance = Vexhood + Vwc - Vairbox;
          var flap_level = get_flap_level(Vexhood, imbalance);
          var flap_scene = ExHoodFlapScenes[flap_level];
          debug("INFO_2", "Level:" + flap_level + ";imbalance:" + imbalance + ";scene:" + flap_scene);

          var levels = JSON.parse(WescoVars.get("Controller/exhood_flap_percentages"));
          WescoVars.save("Controller/ExhoodCurrentValue_flap", levels[flap_level], true);

          tiny.Scene(ExHood_dSID, flap_scene); // send scene to all airboxes
          print_rules_gnuplot(Mode, PrevMode, isWindowOpen, NAirboxes, Vwc, airbox_level, flap_level, Vexhood, Vairbox);
        }
      }
    } //end of function cooking_mode_rules(isWindowOpen, Vwc,Vexhood) {

    function schedule_mode_rules(airbox_level, Vwc) {
      /*Ventilation rules in Schedule mode. This mode has the lowest priority*/
      var BaseVentilationLevel = 0;
      var BaseVentilationActive = WescoVars.get("Controller/BaseVentilation/Enabled");
      if (BaseVentilationActive == true) {
        BaseVentilationLevel = parseInt(WescoVars.get("Controller/BaseVentilation/Level"));
        if (isNaN(BaseVentilationLevel) || (BaseVentilationLevel < 0 || BaseVentilationLevel > 4)) {
          BaseVentilationLevel = 0;
        }
      }
      debug("INFO_3", "BaseVentilationActive:" + BaseVentilationActive);
      debug("INFO_3", "BaseLevel:" + BaseVentilationLevel);


      var level = 0;
      if (!isNaN(airbox_level)) {
        if (airbox_level < BaseVentilationLevel) {
          level = BaseVentilationLevel;
        } else {
          if (airbox_level >= 0 && airbox_level <= 4) {
            level = airbox_level;
          }
        }
      } else {
        level = BaseVentilationLevel;
      }

      debug("INFO_3", "level:" + level);

      var Vairbox = NAirboxes * AirboxLevelVolumes[level];
      var imbalance = Vairbox - Vwc;
      var fan_level = get_fan_level(imbalance);
      var levels = JSON.parse(WescoVars.get("Controller/exhood_volume_levels"));
      var Vexhood = levels[fan_level];
      print_rules_gnuplot(Mode, PrevMode, isWindowOpen, NAirboxes, Vwc, level, 0, Vexhood, Vairbox);

      var fan_scene = ExHoodFanScenes[fan_level];
      tiny.Scene(ExHood_dSID, fan_scene);

      var levels = JSON.parse(WescoVars.get("Controller/exhood_fan_percentages"));
      WescoVars.save("Controller/ExhoodCurrentValue_fan", levels[fan_level], true);

      var airbox_scene = AirboxScenes[level];
      sendAirboxSceneDeferred(airbox_scene, "+5"); // defer to be sent in 5 seconds

      print_rules_gnuplot(Mode, PrevMode, isWindowOpen, NAirboxes, Vwc, level, 0, Vexhood, Vairbox);
    } // end of schedule_mode_rules

    // Processing begins:
    debug("INFO_1", "Processing Air Balance Rules");
    debug("INFO_2", "type:" + type);
    debug("INFO_2", "value:" + value);

    // Global variables 

    var Mode = WescoVars.get("Controller/PresenceState_current");
    var PrevMode = WescoVars.get("Controller/PresenceState_previous");

    debug("INFO_1", "Current mode:" + Mode);
    debug("INFO_1", "Previous mode:" + PrevMode);

    var NAirboxes = airboxes.getActiveDevices();
    var Vwc = wcfans.getVolumeActiveWCFans();
    var isWindowOpen = exhood.getWindowContactStatus();
    var ExHood_dSID = exhood.getFanFlap_dSID();


    /*Handling of different events affecting the air balance state*/

    switch (type) {
      case "UIEvent":
        {
          /*This event is trigger when the user press one of the level buttons
            in the add-on's control tab. In the future, this case can also be
            extended to include a scene coming from a button or the digitalSTROM
            App. The user selection is valid for 1h. After this time, the control 
            goes to the previous mode. See the DeactivateManualMode case in this switch*/

          //case "Scene":
          if (Mode != "Cooking") { // Cooking has prioirity
            if (Mode != "Manual") { //update the mode only if necessary
              debug("INFO_2", "PresentState_current: Schedule");
              debug("INFO_2", "PresentState_previous: Manual");
              WescoVars.save("Controller/PresenceState_previous", "Schedule", false); // save previous mode
              WescoVars.save("Controller/PresenceState_current", "Manual", false); // select "Manual" mode
            }

            if (NAirboxes > 0) {
              WescoVars.save("Controller/CurrentLevelManual", value, true);

              debug("INFO_3", "Value:" + value);

              //set timer to deactivate the manual mode
              var timer = timedevents.timers["TIMER_deactivateManualMode"];
              var parameter = {};
              timedevents.call(timer.name, timer.id, timer.duration, parameter);

              var airbox_level = value;
              schedule_mode_rules(airbox_level, Vwc);
            }
          }
          break;
        }
      case "TimePlan":
        {
          /*This mode has the second highest priority after "Cooking". The time plan
            is bounded by the base ventilation. It is, the base ventilation imposes a 
            lower limit for the ventilation level*/


          // Cooking and Manual have prioirity
          if (Mode == "Schedule") {
            var level = undefined;
            if (value == undefined || (value < 0 || value > 4)) {
              if (BigMode_current == "Absence") {
                level = processSchedule("Absence"); // Absence Schedule
              } else if (BigMode_current == "Presence") {
                level = processSchedule("Presence"); // Presence Schedule
              }
            } else {
              level = value; // argument passed within the balance rules function
            }

            /**************************************************************/
            /**************************************************************/
            /**************************************************************/
            // CONTROL RULE
            // Condition -> Zeitplan mode. Window contact is ignored
            // Action ->
            if (NAirboxes > 0) {
              schedule_mode_rules(level, Vwc);
            }
          } else {
            // Do nothing: the other modes have higher priority
          }
          break;
        }
      case "CookingMode":
        {
          /* This mode has the highest priority*/

          if (value == 0) {

            if (Mode == "Cooking") {
              //cooking off
              // Leave the  Cooking mode
              debug("INFO_1", "Leaving cooking mode");

              WescoVars.save("Controller/PresenceState_current", PrevMode, false); // restore the previous mode
              WescoVars.save("Controller/PresenceState_previous", "Cooking", true); // previous mode is now "Cooking"


              // It is better to turn the flap off while leaving the
              // cooking mode, in this way, we avoid sending two
              // consecutive scenes while controlling the exhood (fan)
              tiny.Scene(ExHood_dSID, ExHoodFlapScenes[0]); //turn the flap off, we don't need it anymore

              var airbox_scene = AirboxScenes[0]; //level 0
              sendAirboxSceneDeferred(airbox_scene, "+5"); // send scene to all airboxes
              print_rules_gnuplot(Mode, PrevMode, isWindowOpen, NAirboxes, Vwc, 0, 0, 0, 0);
            } else {
              // // If the off scene is received when not in cooking mode,
              // it means that the Lüfter was running in ventilation mode
              // and the user was actually trying to turn the exhood on.

              var level = 5; // approx. half power 
              var fan_scene = ExHoodFanScenes[level]; // starts at level 2 approx.
              tiny.Scene(ExHood_dSID, fan_scene);
              WescoVars.save("Controller/PresenceState_previous", Mode, false); // save previous mode
              WescoVars.save("Controller/PresenceState_current", "Cooking", false); // set new mode

              try {
                var levels = JSON.parse(WescoVars.get("Controller/exhood_fan_percentages"));
                WescoVars.save("Controller/ExhoodCurrentValue_fan", levels[level], true);

              } catch (err) {
                debug("WARN_1", "Error setting currentValue_fan:" + err);
                WescoVars.save("Controller/ExhoodCurrentValue_fan", 55, true); //force value to 55%
              }
            }
            // trigger timer to reevaluate the balance rules  
            var timer = timedevents.timers["TIMER_reevaluateBalanceRules"];
            var parameter = {};
            timedevents.call(timer.name, timer.id, timer.duration, parameter);
          } else {
            if (Mode != "Cooking") { //update mode only if necessary
              WescoVars.save("Controller/PresenceState_previous", Mode, false); // save previous mode
              WescoVars.save("Controller/PresenceState_current", "Cooking", true); // set new mode
            }
            var Vexhood = get_exhood_volume(value);
            cooking_mode_rules(isWindowOpen, Vwc, Vexhood);
          }
          break;
        }

      case "WCFan":
        {
          /*An event on the WC fans affects all ventilation modes. The
            balance needs to be readjusted.*/
          if (Mode == "Cooking") {
            var fan_value = WescoVars.get("Controller/ExhoodCurrentValue_fan");
            var Vexhood = get_exhood_volume(fan_value);
            cooking_mode_rules(isWindowOpen, Vwc, Vexhood);
          } else {
            var airbox_level = 0;
            if (Mode == "Manual") {
              airbox_level = WescoVars.get("Controller/CurrentLevelManual");
            } else { //if schedule
              airbox_level = processSchedule("Presence");
            }

            if ((airbox_level == 0 || isNaN(airbox_level)) && Vwc > 0) {
              airbox_level = get_airbox_level(Vwc);
            }
            schedule_mode_rules(airbox_level, Vwc);

          }
          break;
        }

      case "ReevaluateRules":
        {
          /* Re-entrance to evaluating the balance ventilation rules is achieved by
             means of the ReevaluateRules timer.*/
          debug("INFO_2", "Reevaluating rules mode:" + Mode);

          if (Mode == "Manual") {
            var airbox_level = WescoVars.get("Controller/CurrentLevelManual");

            if ((airbox_level == 0 || isNaN(airbox_level)) && Vwc > 0) {
              airbox_level = get_airbox_level(Vwc);
            }
            schedule_mode_rules(airbox_level, Vwc);
          } else if (Mode == "Schedule") {
            var airbox_level = processSchedule("Presence"); // Presence Schedule

            if ((airbox_level == 0 || isNaN(airbox_level)) && Vwc > 0) {
              airbox_level = get_airbox_level(Vwc);
            }
            schedule_mode_rules(airbox_level, Vwc);
          } else if (Mode == "Cooking") {
            var fan_value = WescoVars.get("Controller/ExhoodCurrentValue_fan");
            debug("INFO_3", "Reevaluating rules mode:" + Mode + ";fan_level:" + fan_value);
            var Vexhood = get_exhood_volume(fan_value);
            cooking_mode_rules(isWindowOpen, Vwc, Vexhood);
          }
          break;
        }
      case "DeactivateManualMode":
        {
          /* The Manual mode is deactivated after a timer expires. The controller goes to the
             previous mode.*/
          if (Mode == "Manual") {
            WescoVars.save("Controller/PresenceState_previous", "Manual", false); // save previous mode
            WescoVars.save("Controller/PresenceState_current", "Schedule", true); // select "Manual" mode 
          } else if (PrevMode == "Manual") {
            WescoVars.save("Controller/PresenceState_previous", "Schedule", true); // select "Manual" mode 
          }

          //go back to the schedule or to base ventilation (Grundlüftung)
          var airbox_level = processSchedule("Presence"); // Presence Schedule
          if ((airbox_level == 0 || isNaN(airbox_level)) && Vwc > 0) {
            airbox_level = get_airbox_level(Vwc);
          }
          if (Mode != "Cooking") {
            schedule_mode_rules(airbox_level, Vwc);
          }


          break;
        }

      default:
        break;
    }
  };
};

var control = new Control();