/***************************************************************/
/***************************************************************/
/***************************************************************/
/***************************************************************/
/******* Air ventilation variables ****************************/

var EXHOOD_NFANSCENES = 10;
var EXHOOD_NFLAPSCENES =10;

var ExHoodFanScenes = {
	0: 0,
	1: 1,
	2: 2,
	3: 3,
	4: 4,
	5: 5,
	6: 6,
	7: 7,
	8: 8,
	9: 9
}; // 0,1,2,3,4 -> [0,17,18,19,5];
var AirboxScenes = {
	0: 0,
	1: 5,
	2: 17,
	3: 18,
	4: 19
}; // 0,1,2,3,4 -> [0,17,18,19,5];
var ExHoodFlapScenes = {
	0: 20,
	1: 21,
	2: 22,
	3: 23,
	4: 24,
	5: 25,
	6: 26,
	7: 27,
	8: 28,
	9: 29
}; // 0,1,2,3,4 -> [0,17,18,19,5];

/***************************************************************/
/***************************************************************/
/***************************************************************/
/***************************************************************/

//------------------------------------------------------------
//////// Script constants
var DEFAULT_INT = -1;
var DEFAULT_STRING = '';
var DEFAULT_BOOLEAN = false;
//------------------------------------------------------------

//Extractor hood registers
var DS_POLL_STATUS_INFO = 25; // statusIndex for events comming from the exhood


//------------------------------------------------------------
/////// Airbox constants
var WESCO_AIRBOX_EAN = "7630019804721";
var WESCO_EXTRACTOR_HOOD_EAN = "7630019807753";
var WESCO_WC_FAN_EAN = "7630019807760";

/*Maximum number of times an airbox is read and written*/
var AIRBOX_READ_MAX_RETRY = 3;
var AIRBOX_WRITE_MAX_RETRY = 3;

var FilterReplacementInterval = 365; // in days

var DEFAULT_WCFAN_RUN_TIME = 60; // in seconds -> default Nachlaufzeit
var DEFAULT_WCFAN_VOLUME_RATE = 30; // in m3/h

// dSTiny variables
var DSTINY_REG_BANK = 0x7F;
var AIRBOX_COMMAND_REG = 3;
var AIRBOX_LEVEL_REG = 1;
var FIRMWARE_VERSION_REG = 62;
var LIMMATTOWER_FIRMWARE_VERSION = 0x30;
var AIRBOX_MAX_TIME_BTW_ERRORS = 50000; // seconds
var AIRBOX_CLEARERROR_NTRIES = 5;
var AIRBOX_ERROR_EVENT_STATUS_INDEX = 24;
var AIRBOX_STATUS_EVENT_INDEX = 25;
var AIBOX_LEVEL_REG = 1;
var AIRBOX_LEVEL_NREGS = 3; // number of level registers

var AIRBOX_MINIMUM_LEVEL_REG = 13;
var AIRBOX_MEDIUM_LEVEL_REG = 14;
var AIRBOX_NOMINAL_LEVEL_REG = 15;

// Error codes
var AIRBOX_POWER_ERR = 128;
var AIRBOX_SENSOR_ERR = 130;
var AIRBOX_FAN_ERR = 132;
var AIRBOX_FLAP_ERR = 136;
var AIRBOX_FROST_ERR = 144;
//------------------------------------------------------------



//------------------------------------------------------------
//// Controller constants
var AIRBOX_DEFAULT_LEVEL = 2; // Medium
//------------------------------------------------------------


//------------------------------------------------------------
///////// Debug constants

//var AIRBOX_ERROR_LOG="wesco_airbox_error.log"
var SCRIPT_LOG = "wesco.log"
var DEBUG_LOG = "wesco_debug.log"

var DebugLog = new Logger(DEBUG_LOG);
var WESCO_logger = new Logger(SCRIPT_LOG);

var VerboseModeLevel = 2; // 0 no debug,1,2,3 or 4
//------------------------------------------------------------



// Error codes for the log files
var ErrorCodes = {
	"no_error": "NOERROR",
	"timeout_error": "TMOUTERR",
	"power_error": "POWERERR",
	"sensor_error": "SENSORERR",
	"fan_error": "FANERR",
	"flap_error": "FLAPERR",
	"frost_warning": "FROSTWAR",
	"frost_error": "FROSTERR",
	"deletedAirbox_from_propTree": "AIRBOXLOST",
	"deletedExHood_from_propTree": "EXHOODLOST",
	"deletedWCFan_from_propTree": "WCFANLOST",
	"registeredAirbox_to_propTree": "NEWAIRBOX",
	"registeredWCFan_to_propTree": "NEWWCFAN",
	"registeredExHood_to_propTree": "NEWEXHOOD",
};



function debug(type, message) {
	// Writes
	//Events with type DEBUG are only printed if the variable VerboseMode is true.

	if (VerboseModeLevel > 0) {

		switch (type) {
			case "ERROR":
				if (VerboseModeLevel > 0) {
					DebugLog.logln(";ERROR;" + message);
				}
				break;
			case "INFO_1":
				if (VerboseModeLevel > 1) {
					DebugLog.logln(";INFO;" + message);
				}
				break;
			case "PLOT":
				if (VerboseModeLevel > 1) {
					DebugLog.logln(";PLOT;" + message);
				}
				break;
			case "INFO_2":
				if (VerboseModeLevel > 2) {
					DebugLog.logln(";INFO;" + message);
				}
				break;
			case "INFO_3":
				if (VerboseModeLevel > 3) {
					DebugLog.logln(";INFO;" + message);
				}
				break;
			case "WARN_1":
				if (VerboseModeLevel > 1) {
					DebugLog.logln(";WARN;" + message);
				}
				break;
			case "WARN_2":
				if (VerboseModeLevel > 2) {
					DebugLog.logln(";WARN;" + message);
				}
				break;
			case "WARN_3":
				if (VerboseModeLevel > 1) {
					DebugLog.logln(";WARN;" + message);
				}
				break;
		}
	}//end if
}

function WESCOLog(Type, dsid, ErrorCode, Message) {
	// Type in [E:error,W:warning,I:info]

	// if dsid is 0,  then the error belongs to the script and not to an airbox in particular

	// Message can be emptied
	WESCO_logger.logln(";" + Type + ";" + ErrorCode + ";" + dsid + ";" + Message);
}