var DSTiny = function() {

	this.Scene = function(ID, SceneNum) {
		/*Transmit a scene value to an individually addressed dstiny*/
		var currentDevice = getDevices().byDSID(ID);
		if (currentDevice.isPresent) {
			currentDevice.callScene(parseInt(SceneNum));
			debug("INFO_2", "scene:" + ID + ";scene:" + SceneNum);
		}
	};

	this.writeReg = function(ID, reg, value) {
		var currentDevice = getDevices().byDSID(ID);
		if (currentDevice.isPresent) {
			currentDevice.setConfig(DSTINY_REG_BANK, parseInt(reg), parseInt(value));
			debug("INFO_2", "write:" + ID + ";reg:" + reg + ";value:" + value);
		}
	};

	this.readReg = function(ID, reg) {
		var currentDevice = getDevices().byDSID(ID);
		if (currentDevice.isPresent) {
			var value = currentDevice.getConfig(DSTINY_REG_BANK, parseInt(reg));
			debug("INFO_2", "read:" + ID + ";reg:" + reg + ";value:" + value);
			return value;
		} else {
			return undefined;
		}
	};

	this.writingBusy = function(Nitems) {
		var timer = timedevents.timers["TIMER_remove_busy_flag"];
		var parameter = {};

		var prop_duration = Nitems * 40; // make the timeout proportional
		// to the nombre of parameters
		// written and take into account possible retries.
		if (prop_duration < 60) {
			prop_duration = 60;
		}

		WescoVars.save("Controller/writingInProgress", true, false); // signal that a writing operation is in progress

		// set a timeout to restore the flag after a predefined time
		timedevents.call(timer.name, timer.id, '+' + String(prop_duration), parameter);
	}
	this.writingDone = function() {
		var timer = timedevents.timers["TIMER_remove_busy_flag"];
		WescoVars.save("Controller/writingInProgress", false, false); // signal that a writing operation is over
		timedevents.delete(timer.id); // kill the timer that is supposed to restore the writingInProgressFlag to false
	}


	this.SendMultiple = function(parameter) {
		/*
	  This function implements a FSM that sequentially writes one
	  or several registers (respective scenes) to one or more
	  airboxes.  The operations are done one at the time, with
	  intervals of approx 2 seconds.  The function has to be
	  relaunched every time the timer expires.

	  The relevant arguments for each case are included in the parameter object.
	  The variable parameter.action indicates which kind of action is to be performed.

	  - parameter.action = 'sendScene': a scene is send to all airboxes

	  the scene number is given in parameter.args[0]



	  - paramter.action = 'writeSingle' and parameter.action = 'readSingle' writes/reads a series of
          registers to/from  a single airbox. The airbox is given by
          parameter.dSID and the tuples [register,value] are
          specified in the parameter.args vector parameter.args=[[reg1,value1],[reg2,value2],...[regn,valuen]]


	  - parameter.action = 'writeAll' and 'readAll': similar to
          write/read single Airbox but the registers are written to /read from  all
          airboxes, one at the time, on a daisy-chain.

	  for the readSingle and readAll actions, the args vector can be specified as
	  parameter.args=[reg1,reg2,...regn]

	*/


		var NextFSMState;
		var NextDict;
		var NextDevice;

		// arm the corresponding timer
		var timer = timedevents.timers["TIMER_MultipleRegAction_FSM"];



		function initializeDict(devID, Nitems, Path) {
			try {
				var prop = Property.getNode(Path);
				var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

				var Dict = {};
				var completed = [];
				for (var i = 0; i < Nitems; i++) {
					completed.push(null);
				}

				for (var i in array) {
					var DevPath = array[i].getPath();


					var ID = WescoVars.get(DevPath + "/dSID");
					if (devID != 0) {
						if (ID == devID) {
							Dict[0] = {};
							Dict[0] = {
								'dSID': ID,
								'Path': DevPath,
								'Ntries': 0,
								'completed': completed
							};
						}
					} else { // include all airboxes if ID=0
						Dict[i] = {};
						Dict[i] = {
							'dSID': ID,
							'Path': DevPath,
							'Ntries': 0,
							'completed': completed
						};
					}
				}


				return Dict;
			} catch (err) {
				debug("ERROR", "Error in initializeDict:" + err);
				return {};
			}
		} // end ... function initializeDict(ID,Nitems){



		function cloneDict(Dict) {
			// clone object Dict and return the copy
			var NewDict = {};
			for (prop in Dict) {
				NewDict[prop] = Dict[prop];
			}
			return NewDict;
		} // end ... 	function cloneDict(Dict){


		var path_toDevices = parameter.path_toDevices;
		var Dict = JSON.parse(parameter.Dict);
		var args = JSON.parse(parameter.args);
		var Nitems = parseInt(args.length);

		debug("INFO_3", "state:" + parameter.FSMState);


		//FSM
		switch (parameter.FSMState) {
			case "initialize":
				if (parameter.action == 'writeSingle' || parameter.action == 'readSingle') {
					NextDict = initializeDict(parameter.dSID, Nitems, path_toDevices);
				} else { // all devices
					NextDict = initializeDict(0, Nitems, path_toDevices); // 0 indicates all airboxes
				}
				parameter['Dict'] = JSON.stringify(NextDict);
				parameter['FSMState'] = "doing";
				parameter['Device'] = 0;

				timedevents.call(timer.name, timer.id, timer.duration, parameter);

				this.writingBusy(Nitems); // prevent other writing operations
				break;

			case "doing":

				var ID = Dict[String(parameter.Device)]['dSID'];
				var DevPath = Dict[String(parameter.Device)]['Path'];

				NextDict = cloneDict(Dict);
				var completed = Dict[String(parameter.Device)].completed;
				for (var i = 0; i < Nitems; i++) {

					if (completed[i] === null) { // if the operation with this register has not been completed
						try {
							// write LEVEL register
							if (parameter.action == 'sendScene') {
								var SceneNum = args[0];
								this.Scene(ID, SceneNum);
								NextDict[String(parameter.Device)].Ntries++; // increment the counter of readings
								completed[i] = true; // mark register as ALREADY WRITTEN
								NextDict[String(parameter.Device)].completed = completed;

								break;
							} else if (parameter.action == 'writeSingle' || parameter.action == 'writeAll') {
								var Reg = args[i][0];
								var Value = args[i][1];
								this.writeReg(ID, Reg, Value);
								NextDict[String(parameter.Device)].Ntries++; // increment the counter of readings
								completed[i] = true; // mark register as ALREADY WRITTEN
								NextDict[String(parameter.Device)].completed = completed;
								break;
							} else if (parameter.action == 'readSingle' || parameter.action == 'readAll') {
								var Reg = args[i];
								var Value = this.readReg(ID, Reg);
								NextDict[String(parameter.Device)].Ntries++; // increment the counter of readings
								completed[i] = Value; // mark register as ALREADY WRITTEN
								NextDict[String(parameter.Device)].completed = completed;
								break;
							} else {
								completed[i] = true; // mark register as ALREADY WRITTEN
								NextDict[String(parameter.Device)].completed = completed;
							}
						} catch (err) {
							//WESCOLog("W",ErrorCodes["timeout_error"],ID,"");
							NextDict[String(parameter.Device)].Ntries++; // increment the counter of readings
							debug("ERROR", "exception;" + err + ";" + ID);
						}
					}
				} // end for (var ..


				// Exit condition: The exit condition is when all registers
				// have been written or the max number of tries exceeded

				var AllDone = true;
				var MaxTries = false;


				for (var Dev in Dict) {
					for (var Reg in Dict[Dev].completed) {
						AllDone = AllDone && (Dict[Dev].completed[Reg] !== null);
					}
					MaxTries = MaxTries || (Dict[String(Dev)].Ntries >= AIRBOX_WRITE_MAX_RETRY * Nitems);
					debug("INFO_3", "Ntries:" + Dict[String(Dev)].Ntries);
					debug("INFO_3", "MaxTries:" + MaxTries);
					//If an airbox can not be reached.  No communication!!
					if (Dict[String(Dev)].Ntries >= AIRBOX_WRITE_MAX_RETRY * Nitems) {
						var dsid = Dict[String(Dev)].dSID;
						debug("ERROR", "timeout dstiny");
						WESCOLog("E", ErrorCodes["timeout_error"], dsid, "");
					}
				}


				if (AllDone && !MaxTries) { //If everything was completed
					NextFSMState = "done";
					NextDevice = 100; // an invalid airbox device index

					//mark airbox current level
					if (parameter.path_toDevices == 'Airboxes/Devices') {
						if (parameter.action == 'sendScene') {
							var scene = args[0];
							var level = 0;
							if (scene == AirboxScenes[1]) {
								level = 1;
							} else if (scene == AirboxScenes[2]) {
								level = 2;
							} else if (scene == AirboxScenes[3]) {
								level = 3;
							} else if (scene == AirboxScenes[4]) {
								level = 4;
							}
							WescoVars.save("Controller/CurrentLevel", level, false);
							debug("INFO_1", "Set airbox level to:" + level + ";scene:" + scene);
						}
					}

					// if one of the reading commands is completed, then updated the property tree and status according



					if (parameter.action == 'readSingle' || parameter.action == 'readAll') {
						parameter['Dict'] = JSON.stringify(Dict); // results
					} else {
						parameter['Dict'] = '{}'; //
					}
					parameter['Regs'] = JSON.stringify(args); // registers
					var eventReadingDone = new Event("wesco_RegistersOpCompleted", parameter);
					var eventID = eventReadingDone.raise();


				} else if (MaxTries) { // if the maximum number of tries was reached
					NextFSMState = "done";
					NextDevice = 100; // an invalid airbox device index
				} else {
					NextDevice = parseInt(parameter.Device) + 1;
					if (!(NextDevice in Dict)) {
						NextDevice = 0;
					}
					//if not yet done ... continue writing
					NextFSMState = "doing";
				}

				parameter['Dict'] = JSON.stringify(NextDict);
				parameter['Device'] = NextDevice;
				parameter['FSMState'] = NextFSMState;

				if (parameter.action == 'readSingle' || parameter.action == 'readAll') {
					parameter['args'] = JSON.stringify(args);
				}

				timedevents.call(timer.name, timer.id, timer.duration, parameter);
				break;

			case "done":
				this.writingDone();

				break;
			default:
				NextFSMState = "done";
				this.writingDone();
				//unlockScript();
				break;
		} //end switch
	}; //end ...  function SendMultiple ...



}; // end object DSTiny


var tiny = new DSTiny();