var ExHood = function() {
	this.discover = function() {
		/*
		  This function scans all devices in zone0 at the property tree
		  and retrieves those with EAN number matching WESCO_AIRBOX_EAN.

		  The associated airbox variables at the property tree are
		  retrieved/initialized.
		  */

		function ConfigurationIsValid(path) {
			// Returns true if these fields exist
			try {
				if (WescoVars.get(path + "/dSID") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/errorCode") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/Status") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/Type") == undefined) {
					return false;
				}

				if (WescoVars.get(path + "/errorTime") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/errorFSMState") == undefined) {
					return false;
				}
				return true;
			} catch (err) {
				debug("ERROR", "Exception -> Configuration Invalid: " + path);
				return false;
			}
		} // end 	function ConfigurationIsValid(path){


		function removeNonPresentDevices() {
			/*This function recursevely removes from the property tree
			 * devices that are not anymore present.  This function
			 * does not yet consider subfolders (only one level per
			 * airbox).
			 */

			var exhoodsPath = "ExHood/Devices";

			try {
				var prop = Property.getNode(exhoodsPath);
				var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist


				for (var i in array) { // iterate over the airboxes nodes
					var ExHoodName = array[i].getName();
					var ExHoodPath = array[i].getPath();

					if (!ConfigurationIsValid(ExHoodPath)) { // if device is not present
						var subprop = Property.getNode(ExHoodPath);
						var subarray = subprop.getChildren();

						// iterate over all airbox properties and delete them
						for (var j in subarray) {
							var item = subarray[j].getName();
							subprop.removeChild(item);
						}
						// once all the variables were removed, remove the
						// airbox node.
						prop.removeChild(ExHoodName);

						debug("INFO_1", "Removed exhood device:" + ExHoodId);
					} else {
						try {
							var ExHoodId = WescoVars.get(ExHoodPath + "/dSID");
							var currentExhood = getDevices().byDSID(ExHoodId);

							if (!currentExhood.isPresent) { // if device is not present
								var subprop = Property.getNode(ExHoodPath);
								var subarray = subprop.getChildren();

								// iterate over all airbox properties and delete them
								for (var j in subarray) {
									var item = subarray[j].getName();
									subprop.removeChild(item);
								}
								// once all the variables were removed, remove the
								// airbox node.
								prop.removeChild(ExHoodName);

								debug("INFO_1", "Removed exhood device:" + ExHoodId);
								WESCOLog("W", ErrorCodes["deletedExHood_from_propTree"], ExHoodId, "");
							}
						} catch (err) {
							debug("ERROR", "Exception while removing exhood:" + err);
							var subprop = Property.getNode(ExHoodPath);
							var subarray = subprop.getChildren();

							// iterate over all airbox properties and delete them
							for (var j in subarray) {
								var item = subarray[j].getName();
								subprop.removeChild(item);
							}
							// once all the variables were removed, remove the
							// airbox node.
							prop.removeChild(ExHoodName);

							debug("ERROR", "Removed exhood device:" + ExHoodId);
							WESCOLog("W", ErrorCodes["deletedExHood_from_propTree"], ExHoodId, "");
						}

					}
				} //end for
			} //end try
			catch (err) {
				debug("ERROR", "Exception while removing ExHood:" + err);
				//do nothing. For instance if the directory Devices does not yet exist
			}

		} // end ...     function removeNonPresentDevices(){



		function createDefaultExHood(zone0_path, dsuid, id) {
			// Initializes/retrieves variables corresponding to each
			// airbox.


			try {
				var type = WescoVars.get(zone0_path + "productInfo/PartNumber");
				if (type == '1' || type == '2' || type == '3') { // if one of the two first dstiny configured devices

					var path = "ExHood/Devices/" + dsuid + "/"
					WescoVars.save(path + "dSID", id, false);
					WescoVars.get_create(path + "errorCode", ErrorCodes["no_error"], false);
					WescoVars.get_create(path + "Status", 0x00, false);
					WescoVars.get_create(path + "errorTime", DEFAULT_STRING, false);
					WescoVars.get_create(path + "errorFSMState", "ready", false);
					WescoVars.get_create(path + "CurrentValue_fan", 0, false); // required only if dsidx=1
					WescoVars.get_create(path + "CurrentValue_flap", 0, false); // required only if dsidx=1

					if (type == '1') {
						WescoVars.get_create(path + "Type", "Fan_Flap", false);
					} else if (type == '2') {
						WescoVars.get_create(path + "Type", "Light", false);
					} else if (type == '3') {
						WescoVars.get_create(path + "Type", "Window", false);
					}
				} else {}
				return type;
			} catch (err) {
				debug("ERROR", "Exception while creating new Exhood:" + err);
				// error in the conversion or format
				return undefined;
			}

		} // end ...    function createDefaultExHood(dsuid,id){

		removeNonPresentDevices(); // remove entries of non present devices


		// the list of airboxes is created based on the devices in zone0,
		// whose EAN matches the Airbox one.
		var NExHoods = 0;

		getDevices().perform(function(device) {

			var path = "/apartment/zones/zone0/devices/" + device.dsuid + "/"
			var EAN = WescoVars.get(path + "productInfo/EAN");
			if (EAN == WESCO_EXTRACTOR_HOOD_EAN) {
				NExHoods = NExHoods + 1;
			}

			if (device.isPresent) {
				var dSID = WescoVars.get(path + "dSID");

				// if the EAN number matches the WESCO one, then create an airbox node.
				if (EAN == WESCO_EXTRACTOR_HOOD_EAN) {
					debug("INFO_2", "Create exhood node;EAN:" + EAN + ";dSID:" + dSID + ";dsuid:" + device.dsuid + ";Name:" + device.name);

					// check if the airbox is already at the property tree
					var ExHoodPath = "ExHood/Devices/" + device.dsuid;
					var prop = Property.getNode(ExHoodPath);

					var type = createDefaultExHood(path, device.dsuid, dSID);
					if (prop === undefined || prop === null) {
						if (type == '1') {
							debug("INFO_2", "Create extractor hood node ;EAN:" + EAN + ";dSID:" + dSID + ";dsuid:" + device.dsuid);
							WESCOLog("I", ErrorCodes["registeredExHood_to_propTree"], dSID, "");
						}
					}
				}
			}
		});

		WescoVars.store();
		debug("INFO_2", "Completed extractor hood  discovery");
	};


	this.isExHood = function(dsuid) {
		/*
		  This function verifies wheather a dsuid belongs to an airbox or not.
		  The function does not verify if the device is present or not.
		  */
		var ExHoodPath = "ExHood/Devices/" + dsuid;
		var prop = Property.getNode(ExHoodPath);

		if (prop) {
			return true;
		} else {
			return false;
		}
	};

	this.getWindowContactStatus = function() {
		/*
	  Returns true if the  window contact is open (if there is a window contact sensor)
	  */
		var path = "ExHood/Devices";
		try {
			var prop = Property.getNode(path);
			var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

			for (var i in array) { // iterate over the airboxes nodes
				var DevPath = array[i].getPath();
				var Type = WescoVars.get(DevPath + "/Type");
				if (Type == "Window") {
					var Status = parseInt(WescoVars.get(DevPath + "/Status"));
					if (Status > 0) {
						return true; // window open
					}
				}
			}
			return false;
		} catch (err) {
			debug("ERROR", "Exception while reading Exhood window contact status:" + err);
			return false;
		}

	};

	this.updateVariables = function(Path, Reg, Value) {

		debug("INFO_3", "path:" + Path + ";Reg:" + Reg + ";Value:" + Value);
		if (Value === null || Path.indexOf("ExHood") < 0) {
			return;
		}

		switch (Reg) {
			// exhood fan % levels
			case 16:
			case 17:
			case 18:
			case 19:
			case 20:
			case 21:
			case 22:
			case 23:
			case 24:
			case 25:
				var levels = WescoVars.get("Controller/exhood_fan_percentages");
				levels = JSON.parse(levels);
				levels[parseInt(Reg) - 16] = parseInt(Value);
				WescoVars.save("Controller/exhood_fan_percentages", JSON.stringify(levels), false);
				break;
				// flap % levels
			case 26:
			case 27:
			case 28:
			case 29:
			case 30:
			case 31:
			case 32:
			case 33:
			case 34:
			case 35:
				var levels = WescoVars.get("Controller/exhood_flap_percentages");
				levels = JSON.parse(levels);
				levels[parseInt(Reg) - 26] = parseInt(Value);
				WescoVars.save("Controller/exhood_flap_percentages", JSON.stringify(levels), false);
				break;
			default:
				break;
		} // end switch(Reg)
	}; // end function updateVariables(Path,Reg,Value){


	this.getFanFlap_dSID = function() {
		/*
	  Returns true if the  window contact is open (if there is a window contact sensor)
	  */
		var path = "ExHood/Devices";
		try {
			var prop = Property.getNode(path);
			var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

			for (var i in array) { // iterate over the airboxes nodes
				var DevPath = array[i].getPath();
				var Type = WescoVars.get(DevPath + "/Type");
				if (Type == "Fan_Flap") {
					var dsid = WescoVars.get(DevPath + "/dSID");
					return dsid;
				}
			}
			debug("WARN_1", "Fan_Flap_dSID not found");
			return undefined;
		} catch (err) {
			debug("ERROR", "Exception while getting ExHood Fan_Flap device dSID:" + err);
			return undefined;
		}

	};


}; // end object ExHood


var exhood = new ExHood();