var WCFans = function() {


	this.discover = function() {
		/*
		  This function scans all devices in zone0 at the property tree
		  and retrieves those with EAN number matching WESCO_AIRBOX_EAN.

		  The associated airbox variables at the property tree are
		  retrieved/initialized.
		*/

		function ConfigurationIsValid(path) {
			// Returns true if these fields exist
			try {
				if (WescoVars.get(path + "/dSID") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/dSUID") == undefined) {
					return false;
				}

				if (WescoVars.get(path + "/ZoneID") == undefined) {
					return false;
				}

				if (WescoVars.get(path + "/VolumeRate") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/Status") == undefined) {
					return false;
				}
				if (WescoVars.get(path + "/OnTime") == undefined) {
					return false;
				}
				return true;
			} catch (err) {
				debug("WARN_1", "WCFan Configuration Invalid: " + path);
				return false;
			}
		} // end 	function ConfigurationIsValid(path){


		function removeNonPresentDevices() {
			/*This function recursevely removes from the property tree
			 * devices that are not anymore present.  This function
			 * does not yet consider subfolders (only one level per
			 * airbox).
			 */

			var WCFansPath = "WCFans/Devices";

			try {
				var prop = Property.getNode(WCFansPath);
				var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist


				for (var i in array) { // iterate over the airboxes nodes
					var WCFansName = array[i].getName();
					var WCFansPath = array[i].getPath();

					if (!ConfigurationIsValid(WCFansPath)) { // if device is not present
						var subprop = Property.getNode(WCFansPath);
						var subarray = subprop.getChildren();

						// iterate over all airbox properties and delete them
						for (var j in subarray) {
							var item = subarray[j].getName();
							subprop.removeChild(item);
						}
						// once all the variables were removed, remove the
						// airbox node.
						prop.removeChild(WCFansName);

						debug("INFO_1", "Removed WCFan device:" + WCFansId);
					} else {
						try {
							var WCFansId = WescoVars.get(WCFansPath + "/dSID");
							var currentWCFan = getDevices().byDSID(WCFansId);

							if (!currentWCFan.isPresent) { // if device is not present
								var subprop = Property.getNode(WCFansPath);
								var subarray = subprop.getChildren();

								// iterate over all airbox properties and delete them
								for (var j in subarray) {
									var item = subarray[j].getName();
									subprop.removeChild(item);
								}
								// once all the variables were removed, remove the
								// airbox node.
								prop.removeChild(WCFansName);

								debug("INFO_1", "Removed WCFan device:" + WCFansId);
								WESCOLog("W", ErrorCodes["deletedWCFan_from_propTree"], WCFansId, "");
							}
						} catch (err) { // if the device can not be accessed
							debug("WARN_1", "Exception while  removing WCFan:" + err);
							var subprop = Property.getNode(WCFansPath);
							var subarray = subprop.getChildren();

							// iterate over all airbox properties and delete them
							for (var j in subarray) {
								var item = subarray[j].getName();
								subprop.removeChild(item);
							}
							// once all the variables were removed, remove the
							// airbox node.
							prop.removeChild(WCFansName);


							debug("INFO_1", "Removed WCFan device:" + WCFansId);
						}
					}
				} //end for
			} //end try
			catch (err) {
				debug("WARN_1", "Exception while removing WCFan " + err);
				//do nothing. For instance if the directory Devices does not yet exist
			}

		} // end ...     function removeNonPresentDevices(){



		function createDefaultWCFans(zone0_path, dsuid, id, ZoneID) {
			// Initializes/retrieves variables corresponding to each
			// airbox.

			try {
				var path = "WCFans/Devices/" + dsuid + "/"
				WescoVars.save(path + "dSID", id, false);
				WescoVars.save(path + "dSUID", dsuid, false);
				WescoVars.save(path + "ZoneID", ZoneID, false);
				WescoVars.get_create(path + "OnTime", DEFAULT_WCFAN_RUN_TIME, false);
				WescoVars.get_create(path + "Status", 0x00, false);
				WescoVars.get_create(path + "VolumeRate", DEFAULT_WCFAN_VOLUME_RATE, false);
			} catch (err) {
				debug("WARN_1", "Exception while creating WCFan" + err);
				// error in the conversion or format
			}

		} // end ...    function createDefaultWCFans(dsuid,id){

		removeNonPresentDevices(); // remove entries of non present devices


		// the list of airboxes is created based on the devices in zone0,
		// whose EAN matches the Airbox one.
		var NWCFanss = 0;

		getDevices().perform(function(device) {

			var path = "/apartment/zones/zone0/devices/" + device.dsuid + "/"
			var EAN = WescoVars.get(path + "productInfo/EAN");
			if (EAN == WESCO_WC_FAN_EAN) {
				NWCFanss = NWCFanss + 1;
			}

			if (device.isPresent) {
				var dSID = WescoVars.get(path + "dSID");
				var ZoneID = WescoVars.get(path + "ZoneID");

				// if the EAN number matches the WESCO one, then create an airbox node.
				if (EAN == WESCO_WC_FAN_EAN) {
					debug("INFO_2", "Create wcfan node;EAN:" + EAN + ";dSID:" + dSID + ";dsuid:" + device.dsuid + ";Name:" + device.name);

					// check if the airbox is already at the property tree
					var WCFansPath = "WCFans/Devices/" + device.dsuid;
					var prop = Property.getNode(WCFansPath);

					createDefaultWCFans(path, device.dsuid, dSID, ZoneID);
					if (prop === undefined || prop === null) {
						debug("INFO_2", "Create WC Fan node ;EAN:" + EAN + ";dSID:" + dSID + ";dsuid:" + device.dsuid);
						WESCOLog("I", ErrorCodes["registeredWCFan_to_propTree"], dSID, "");
					}

				}
			}
		});

		WescoVars.store();
		debug("INFO_2", "Completed wc fans  discovery");
	};

	this.setRegs = function(dsuid, dict) {

		try {
			var path = "WCFans/Devices/" + dsuid;
			var Regs = JSON.parse(dict);
			for (var v in Regs) {
				WescoVars.save(path + "/" + v, Regs[v], false);
			}
			WescoVars.store();
			debug("INFO_3", "Parameters save to WCFan:" + dsuid);

			if (Regs.OnTime !== undefined && Regs.OnTime !== null) { // only if the OnTime was written
				this.confSceneResponder(dsuid);
			}
		} catch (err) {
			debug("WARN_1", "Exception while setting WCFan vars" + err);
		}
	};

	this.getVolumeActiveWCFans = function() {
		/*Returns the volume of all WCFans active at the moment*/

		var WCFansPath = "WCFans/Devices";
		var volume = 0; // value to be returned

		try {
			var prop = Property.getNode(WCFansPath);
			var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

			for (var i in array) { // iterate over the airboxes nodes
				var DevPath = array[i].getPath();
				var Status = parseInt(WescoVars.get(DevPath + "/Status"));
				if (Status > 0) {
					var wc_volume = parseInt(WescoVars.get(DevPath + "/VolumeRate"));
					volume = volume + wc_volume;
				}
			}
			return volume;
		} catch (err) {
			debug("WARN_1", "Exception while reading WC status and  volume:" + err);
			return volume;
		}
	};

	this.setWCStatus = function(dsuid, state) {
		var path = "WCFans/Devices/";
		try {
			if (state == "active") {
				WescoVars.save(path + dsuid + "/Status", 1, true);
				debug("INFO_2", "Setting Status of WCFan:" + dsuid + " to 1");
			} else {
				WescoVars.save(path + dsuid + "/Status", 0, true);
				debug("INFO_2", "Setting Status of  WCFan:" + dsuid + " to 0");
			}
		} catch (err) {
			debug("WARN_1", "Exception while updating WC status:" + err);
		}
	};

	this.isWCFans = function(dsuid) {
		/*
		  This function verifies wheather a dsuid belongs to an airbox or not.
		  The function does not verify if the device is present or not.
		*/
		var WCFansPath = "WCFans/Devices/" + dsuid;
		var prop = Property.getNode(WCFansPath);

		if (prop) {
			return true;
		} else {
			return false;
		}
	};


	this.createSceneResponderEntry = function(ZoneID, klemme_dsuid, duration) {


		function create_trigger(ZoneID, dsuid, triggerNum, scene, state) {
			// This function is triggered when the given scene on the
			// given zone in group 10 is received. The resulting
			// action is to produce the wesco-wcfan-state-change-event

			var path = "WCFans/Devices/" + dsuid + "/triggers/" + triggerNum;
			WescoVars.save(path + "/id", "WescoWCFanStateChange_" + dsuid + "_" + triggerNum, false);
			WescoVars.save(path + "/name", "WescoWCFanStateChange", false);

			//conditions
			WescoVars.save(path + "/conditions/enabled", true, false);
			//WescoVars.save(path + "/conditions/systemState/fire", 2, false);
			WescoVars.save(path + "/conditions/systemState/name", "fire", false);
			WescoVars.save(path + "/conditions/systemState/value", "2", false);

			//actions

			//triggers
			WescoVars.save(path + "/triggers/" + triggerNum + "/type", "zone-scene", false);
			WescoVars.save(path + "/triggers/" + triggerNum + "/zone", ZoneID, false);
			WescoVars.save(path + "/triggers/" + triggerNum + "/group", 10, false);
			WescoVars.save(path + "/triggers/" + triggerNum + "/scene", scene, false);

			WescoVars.save(path + "/triggers/scope", "wesco", false);
			WescoVars.save(path + "/triggers/delay", 0, false);
			WescoVars.save(path + "/triggers/persistentScope", false, true);


			var event = {};
			event["dsuid"] = dsuid;
			event["state"] = state;
			event["scene"] = scene;

			registerTrigger("/scripts/wesco-balance/" + path, "wesco-wcfan-state-change-event", event);

		}

		create_trigger(ZoneID, klemme_dsuid, 0, 14, "active"); //ZoneID, dsuid, triggerNum, scene, state
		create_trigger(ZoneID, klemme_dsuid, 1, 13, "inactive");


		//registerState("WescoEvent",true); //-> um die State anzulegen
		//unregisterState("WescoEvent"); -> um die State zu loeschen

		/* Creates SR entries for a given Klemme for both the light on
		 * and off events*/
		var triggerDefs_on = [{
			type: "state-change",
			name: "zone." + ZoneID + ".light",
			state: "active"
		}];

		var triggerDefs_off = [{
			type: "state-change",
			name: "zone." + ZoneID + ".light",
			state: "inactive"
		}];


		//http://redmine.digitalstrom.org/projects/dss/wiki/Structure_of_a_Property_-_Action
		var actionDefs_on = [{
				type: "zone-scene", // eventually replace by "zone-scene (parameters type, scene, group, force (boolean), delay"
				zone: ZoneID,
				scene: 14,
				group: 10,
				delay: 0,
			},
			// 		    {
			//     type:"change-addon-state",
			//     statename:"WescoEvent", // Klemme
			//     state:"active",
			//     delay:0,
			//     addonId:"wesco",
			// },
		];

		var actionDefs_off = [{
				type: "zone-scene",
				zone: ZoneID,
				scene: 13,
				group: 10,
				delay: 0,
			},
			// 		     {
			// type:"change-addon-state",
			// statename:"WescoEvent", // Klemme
			// state:"inactive",
			// delay:0,
			// addonId:"wesco",
			// 		     }
		];

		var oSRObject_on = {
			id: 'wesco_on_' + klemme_dsuid,
			name: 'Wesco Auto Ventilation On',
			triggers: triggerDefs_on,
			actions: actionDefs_on,
			conditions: {
				enabled: true,
				systemState:[{name:'fire',value:"2"}], //fire is inactive
			},
			delay: 5,
			scope: "wesco",
			persistentScope: false
		};

		var oSRObject_off = {
			id: 'wesco_off_' + klemme_dsuid,
			name: 'Wesco Auto Ventilation Off',
			triggers: triggerDefs_off,
			actions: actionDefs_off,
			conditions: {
				enabled: true
			},
			delay: duration,
			scope: "wesco",
			persistentScope: false
		};

		// configure SR event for light on
		new Event("system-addon-scene-responder.config", {
			actions: 'save',
			value: escape(JSON.stringify(oSRObject_on))
		}).raise();
		// configure SR event for light off
		new Event("system-addon-scene-responder.config", {
			actions: 'save',
			value: escape(JSON.stringify(oSRObject_off))
		}).raise();
		debug("INFO_2", "Configured scene responder for parameters ZoneID:" + ZoneID + ";dsuid:" + klemme_dsuid + ";OnTime:" + duration);
	}; // end function create scene responder entry


	this.confSceneResponderAllWCFans = function() {
		try {
			var prop = Property.getNode("WCFans/Devices/");
			var array = prop.getChildren(); // vector of airboxes indexes, empty if the node does not exist

			for (var i in array) {
				var DevPath = array[i].getPath();
				var ZoneID = WescoVars.get(DevPath + "/ZoneID");
				var OnTime = parseInt(WescoVars.get(DevPath + "/OnTime"));
				var dsuid = WescoVars.get(DevPath + "/dSUID");
				this.createSceneResponderEntry(ZoneID, dsuid, OnTime);
			}
		} catch (err) {
			debug("ERROR", "Error confSceneResponderAllWCFans:" + err);
		}
	};


	this.confSceneResponder = function(dsuid) {

		try {
			var DevPath = "WCFans/Devices/" + dsuid;
			var ZoneID = WescoVars.get(DevPath + "/ZoneID");
			var OnTime = parseInt(WescoVars.get(DevPath + "/OnTime"));
			this.createSceneResponderEntry(ZoneID, dsuid, OnTime);
		} catch (err) {
			debug("WARN_1", "Exception in wcfan confSceneResponder: " + err);
			// error in the conversion or format
		}

	};



}; // end of WCFans object

var wcfans = new WCFans();