/*
  controllers.js
  Angular module that provides the main and only controller for the wesco airbox app
  */


var airboxControllers = angular.module('airboxControllers', ['dSTinyDSS', 'dSS']);

airboxControllers.controller(

	'AirboxConfCtrl', ['$q', '$scope', '$rootScope', '$sce', '$modal', '$timeout', 'dSTinyDSS', 'dss', 'gettextCatalog', '$confirm',
		function($q, $scope, $rootScope, $sce, $modal, $timeout, dSTinyDSS, dss, gettextCatalog, $confirm) {

			// Ajax request states to show/hide elements depending on if content is loaded
			$scope.ajax = {
				'BusyFlag': false,
				'ButtonsLocked': false,
				'SceneButtonsLocked': false,
				'TabsLocked': false,
				'OpCompletedSubscriptionId': 47,
			};

			var MAXIMUM_EXHOOD_VOLUME = 1000; //m3/h
			var MAXIMUM_AIRBOX_VOLUME = 150; //m3/h

			// Variables for authentication into advanced configuration mode
			$scope.auth = {
					user: '',
					password: '',
					_user: 'admin',
					_password: 'wesco5430'
				}
				// Initialize variables database
			$scope.tiny_conf_vars = {};

			dSTinyDSS.getAirboxes()
				.then(function(airboxes) {
					if (airboxes) {
						dSTinyDSS.getAirboxVolumeLevels()
							.then(function(levels) {
								$scope.airbox_levels = levels;
								//$scope.exhood_fan_levels=levels;
							});

						$scope.airbox_vars = Airbox_GetVars(); // for configuring the airbox levels
						for (var i = 0; i < airboxes.length; i++) {
							var description = airboxes[i].Description;
							var dsid = airboxes[i].dSID;
							var key = "k_" + airboxes[i].dSID;
							//$scope.tiny_conf_vars.push({"name":dsid,"dsid":dsid,"description":description,"variables":Airbox_GetVars()});

							$scope.tiny_conf_vars[key] = {
								"type": "Airbox",
								"name": dsid,
								"dsid": dsid,
								"description": description,
								"variables": Airbox_GetVars(),
								"all_selected": false,
								"gen_reg": 0,
								"gen_reg_value": 0
							};
						}
					};
					dss.Property.getString("/system/language/locale").then(function(locale) {
						gettextCatalog.setCurrentLanguage(locale.substr(0, 2));
					});

				})
				.then(function() {
					$scope.varindex = {};
					for (var ip in $scope.tiny_conf_vars) { // variable cathegories
						if ($scope.tiny_conf_vars[ip].type == "Airbox") {
							for (var iv in $scope.tiny_conf_vars[ip].variables) { // variables in each cathegory
								var v = $scope.tiny_conf_vars[ip].variables[iv];
								// if($scope.varindex[v.name]) {
								//     console.log("WARNING: Multiple variables with name "+v.name+" found in definition file.");
								// }
								//$scope.varindex[v.name] = v; // store references to access by name
								v.selected = false;
								if (typeof v.uservalue === 'undefined') {
									v.uservalue = v.default;
								}
							}
						} //end if
					}
				}, function error() {
					console.log("Airbox array is empty")
					$scope.tiny_conf_vars = {};
				});

			dSTinyDSS.getExHoods()
				.then(function(exhoods) {
					if (exhoods) {

						dSTinyDSS.getExHoodVolumeLevels()
							.then(function(levels) {
								$scope.exhood_levels = levels;
							});

						for (var i = 0; i < exhoods.length; i++) {
							if (exhoods[i].Type == 'Fan_Flap') {
								var description = exhoods[i].Description;
								var dsid = exhoods[i].dSID;
								var key = "k_" + exhoods[i].dSID;
								//$scope.tiny_conf_vars.push({"name":dsid,"dsid":dsid,"description":description,"variables":Airbox_GetVars()});
								$scope.tiny_conf_vars[key] = {
									"type": "ExHood",
									"name": dsid,
									"dsid": dsid,
									"description": description,
									"variables": ExHood_GetVars(),
									"all_selected": false
								};
							}
						}
					};
					// dss.Property.getString("/system/language/locale").then(function(locale) {
					// gettextCatalog.setCurrentLanguage(locale.substr(0,2));
					// });

				})
				.then(function() {
					$scope.varindex = {};
					for (var ip in $scope.tiny_conf_vars) { // variable cathegories
						if ($scope.tiny_conf_vars[ip].type == "ExHood") {
							for (var iv in $scope.tiny_conf_vars[ip].variables) { // variables in each cathegory
								var v = $scope.tiny_conf_vars[ip].variables[iv];
								v.selected = false;
								if (typeof v.uservalue === 'undefined') {
									v.uservalue = v.default;
								}
							}
						}
					};
				}, function error() {
					console.log("ExHood array is empty")
					$scope.tiny_conf_vars = {};
				});



			dSTinyDSS.getWCFans()
				.then(function(wcfans) {
					if (wcfans) {
						for (var i = 0; i < wcfans.length; i++) {
							var description = wcfans[i].Description;
							var dsid = wcfans[i].dSID;
							var dsuid = wcfans[i].dSUID;
							var key = "k_" + wcfans[i].dSID;
							$scope.tiny_conf_vars[key] = {
								"type": "WCFan",
								"name": dsid,
								"dsid": dsid,
								"dsuid": dsuid,
								"description": description,
								"variables": WCFans_GetVars(),
								"all_selected": false
							};

							dss.Scene.setJokerGroup(dsid, 10); // set Klemme to group 10 (ventilation)
							//console.log("set klemme:"+dsuid+" to group 10");
							//$scope.tiny_conf_vars[key]={"type":"WCFan"tin,"name":dsid,"dsid":dsid,"description":description};
						}
					};
					// dss.Property.getString("/system/language/locale").then(function(locale) {
					// gettextCatalog.setCurrentLanguage(locale.substr(0,2));
					// });

				})
				.then(function() {
					$scope.varindex = {};
					for (var ip in $scope.tiny_conf_vars) { // variable cathegories
						if ($scope.tiny_conf_vars[ip].type == "WCFan") {
							for (var iv in $scope.tiny_conf_vars[ip].variables) { // variables in each cathegory
								var v = $scope.tiny_conf_vars[ip].variables[iv];
								v.selected = false;
								if (typeof v.uservalue === 'undefined') {
									v.uservalue = v.default;
								}
							}
						}
					};
				}, function error() {
					console.log("WCFan array is empty")
					$scope.tiny_conf_vars = {};
				});



			$scope.enterAuth = function() {
				if ($scope.auth.user == $scope.auth._user && $scope.auth.password == $scope.auth._password) {
					$rootScope.advancedConf = true;
				}
			};
			$scope.leaveAuth = function() {
				$rootScope.advancedConf = false;
				$scope.auth.user = '';
				$scope.auth.password = '';
			};

			$scope.getAuth = function() {
				if ($rootScope === undefined) {
					return false;
				} else {
					return $rootScope.advancedConf;
				}
			}

			$scope.selectAll = function(dsid) {
				// Select all configuration registers for a given airbox

				var key = "k_" + dsid;
				var all_selected = $scope.tiny_conf_vars[key].all_selected;
				$scope.tiny_conf_vars[key].all_selected = !all_selected;
				for (var iv in $scope.tiny_conf_vars[key].variables) { // variables in each cathegory
					var v = $scope.tiny_conf_vars[key].variables[iv];
					v.selected = !all_selected;
				}
			};

			$scope.writeAirboxVolumeLevels = function() {
				dSTinyDSS.setAirboxVolumeLevels($scope.airbox_levels).then(function() {
					//alert("OK");
				});
			}

			$scope.readSelectedRegisters = function(dsid) {
				// Read selected registers on the corresponding aribox

				var title;
				var text;

				failed = function() {
					title = gettextCatalog.getString("Error")
					text = gettextCatalog.getString("The reading operation could not be completed! Please verify the connections and retry later.");
					$confirm({
						text: text,
						title: title,
						ok: 'OK',
						cancel: 'Cancel'
					});
					$scope.ajax.ButtonsLocked = false;
					$scope.ajax.TabsLocked = false;
				}

				dSTinyDSS.getBusyFlag().then(function(Busy) {
					if (Busy) {
						title = gettextCatalog.getString("Channel is busy")
						text = gettextCatalog.getString("Reading or writing operation in progress, please retry later.")
						$confirm({
								text: text,
								title: title,
								ok: 'OK',
								cancel: 'Cancel'
							})
							.then(function() {
								return;
							});
						return;
					}

					var key = "k_" + dsid;
					var regs = []; // choose the selected (checked) registers


					for (var iv in $scope.tiny_conf_vars[key].variables) {
						var v = $scope.tiny_conf_vars[key].variables[iv];
						if (v.selected) {
							regs.push(v.reg); // this registers contains the checked variables' registers
						}
					}

					if (regs.length > 0) {
						var estimated_time = 5 * regs.length;
						var eventTimeout = 8 * estimated_time * 1000; //ms

						title = gettextCatalog.getString("Do you want to continue?");
						text = gettextCatalog.getString("This operation might take a while (approx: {{seconds}} s). Please do not refresh or close this page.", {
							seconds: estimated_time
						});
						$confirm({
								text: text,
								title: title,
								ok: 'OK',
								cancel: 'Cancel'
							})
							.then(function() {
								$scope.ajax.ButtonsLocked = true;
								$scope.ajax.TabsLocked = true;
								return dss.Event.unsubscribe('wesco_RegistersOpCompleted', $scope.ajax.OpCompletedSubscriptionId)
									.finally(function() {
										return dss.Event.subscribe('wesco_RegistersOpCompleted', $scope.ajax.OpCompletedSubscriptionId)
											.then(function() {
												var type = $scope.tiny_conf_vars[key].type;
												return dSTinyDSS.readRegs(dsid, regs, type).then(function(result) {
													return dss.Event.get($scope.ajax.OpCompletedSubscriptionId, eventTimeout)
														.then(function(result) {
															$scope.ajax.ButtonsLocked = false;
															$scope.ajax.TabsLocked = false;
															if (result.events[0] !== undefined) {
																if (result.events[0].properties !== undefined) {
																	var Dict = JSON.parse(result.events[0].properties.Dict);
																	var regs = JSON.parse(result.events[0].properties.Regs);
																	if (Dict[0]) {
																		var values = Dict[0].completed;
																	}
																	if (values !== undefined && regs !== undefined) {
																		if (values.length == regs.length && values.length > 0) {
																			// update variables with returned values
																			for (var i = 0; i < regs.length; i++) {
																				var var_index = "REG_" + regs[i];
																				if ($scope.tiny_conf_vars[key].variables[var_index].unit == '%') {
																					if ($scope.tiny_conf_vars[key].type == 'Airbox') {
																						$scope.tiny_conf_vars[key].variables[var_index].uservalue = values[i] - 100;
																					} else {
																						$scope.tiny_conf_vars[key].variables[var_index].uservalue = values[i];
																					}
																				} else if (typeof $scope.tiny_conf_vars[key].variables[var_index].values !== 'undefined') { // Selectable values require strings
																					$scope.tiny_conf_vars[key].variables[var_index].uservalue = String(values[i]);
																				} else {
																					$scope.tiny_conf_vars[key].variables[var_index].uservalue = values[i];
																				}
																			}
																			title = gettextCatalog.getString("Success")
																			text = gettextCatalog.getString("{{N}} registers successfully read from device {{dsid}}!", {
																				N: values.length,
																				dsid: dsid
																			});
																			$confirm({
																				text: text,
																				title: title,
																				ok: 'OK',
																				cancel: 'Cancel'
																			});
																		}
																	} else {
																		failed();
																	}
																} // end  if (result.events[0].properties!==undefined){
															} else {
																failed();
															}
														})
														.catch(function() {
															failed();
														});
												});
											})
											.catch(function() {
												failed();
											});
									}).catch(function() {
										failed();
									});
							});
					} // end if regs.length
				});
			}; // end function readSelectedRegisters


			$scope.readGenericRegister = function(dsid) {
				// Read selected registers on the corresponding aribox

				var title;
				var text;

				failed = function() {
					title = gettextCatalog.getString("Error")
					text = gettextCatalog.getString("The reading operation could not be completed! Please verify the connections and retry later.");
					$confirm({
						text: text,
						title: title,
						ok: 'OK',
						cancel: 'Cancel'
					});
					$scope.ajax.ButtonsLocked = false;
					$scope.ajax.TabsLocked = false;
				}

				dSTinyDSS.getBusyFlag().then(function(Busy) {
					if (Busy) {
						title = gettextCatalog.getString("Channel is busy")
						text = gettextCatalog.getString("Reading or writing operation in progress, please retry later.")
						$confirm({
								text: text,
								title: title,
								ok: 'OK',
								cancel: 'Cancel'
							})
							.then(function() {
								return;
							});
						return;
					}

					var key = "k_" + dsid;
					var regs = [];
					regs.push($scope.tiny_conf_vars[key].gen_reg);

					var estimated_time = 5;
					var eventTimeout = 8 * estimated_time * 1000; //ms

					title = gettextCatalog.getString("Do you want to continue?");
					text = gettextCatalog.getString("This operation might take a while (approx: {{seconds}} s). Please do not refresh or close this page.", {
						seconds: estimated_time
					});
					$confirm({
							text: text,
							title: title,
							ok: 'OK',
							cancel: 'Cancel'
						})
						.then(function() {
							$scope.ajax.ButtonsLocked = true;
							$scope.ajax.TabsLocked = true;
							return dss.Event.unsubscribe('wesco_RegistersOpCompleted', $scope.ajax.OpCompletedSubscriptionId)
								.finally(function() {
									return dss.Event.subscribe('wesco_RegistersOpCompleted',
											$scope.ajax.OpCompletedSubscriptionId)
										.then(function() {
											var type = $scope.tiny_conf_vars[key].type;
											return dSTinyDSS.readRegs(dsid, regs, type).then(function(result) {
												return dss.Event.get($scope.ajax.OpCompletedSubscriptionId, eventTimeout)
													.then(function(result) {
														$scope.ajax.ButtonsLocked = false;
														$scope.ajax.TabsLocked = false;
														if (result.events[0] !== undefined) {
															if (result.events[0].properties !== undefined) {
																var Dict = JSON.parse(result.events[0].properties.Dict);
																var regs = JSON.parse(result.events[0].properties.Regs);
																if (Dict[0]) {
																	var values = Dict[0].completed;
																}
																if (values !== undefined && regs !== undefined) {
																	if (values.length == regs.length && values.length > 0) {
																		// update variables with returned values
																		for (var i = 0; i < regs.length; i++) {
																			var var_index = "REG_" + regs[i];
																			if ($scope.tiny_conf_vars[key].type == 'Airbox') {
																				$scope.tiny_conf_vars[key].gen_reg_value = values[i];
																			}
																			title = gettextCatalog.getString("Success")
																			text = gettextCatalog.getString("{{N}} registers successfully read from device {{dsid}}!", {
																				N: values.length,
																				dsid: dsid
																			});
																			$confirm({
																				text: text,
																				title: title,
																				ok: 'OK',
																				cancel: 'Cancel'
																			});
																		}
																	}
																} else {
																	failed();
																}
															} // end  if (result.events[0].properties!==undefined){
														} else {
															failed();
														}
													})
													.catch(function() {
														failed();
													});
											});
										})
										.catch(function() {
											failed();
										});
								}).catch(function() {
									failed();
								});
						});
				});
			}; // end function readGenericRegister



			$scope.readSelectedWCFanRegisters = function(dsid) {
				// Read selected registers on the corresponding wcfan

				var key = "k_" + dsid;


				var variables = $scope.tiny_conf_vars[key].variables;
				var type = $scope.tiny_conf_vars[key].type;
				var dsuid = $scope.tiny_conf_vars[key].dsuid;

				return dSTinyDSS.getWCFan(dsuid).then(function(result) {
					//hardcoded registers

					//VolumeRate
					if (variables["VolumeRate"].selected) {
						$scope.tiny_conf_vars[key].variables["VolumeRate"].uservalue = result.VolumeRate;
					}

					//OnTime
					if (variables["OnTime"].selected) {
						$scope.tiny_conf_vars[key].variables["OnTime"].uservalue = result.OnTime;
					}
				});

			}; // end function readSelectedWCFansRegisters

			$scope.writeSelectedWCFanRegisters = function(dsid) {
				// Read selected registers on the corresponding aribox

				var key = "k_" + dsid;
				var dict = {};

				var variables = $scope.tiny_conf_vars[key].variables;
				var type = $scope.tiny_conf_vars[key].type;
				var dsuid = $scope.tiny_conf_vars[key].dsuid;


				// hardcode registers
				if (variables["VolumeRate"].selected && type == 'WCFan') {
					dict["VolumeRate"] = variables["VolumeRate"].uservalue;
				}

				if (variables["OnTime"].selected && type == 'WCFan') {
					dict["OnTime"] = variables["OnTime"].uservalue;
				}

				if (Object.keys(dict).length > 0) {
					dSTinyDSS.writeWCFanRegs(dsuid, dict);
				}


			}; // end function readSelectedWCFansRegisters



			$scope.checkifBusy = function() {
				// check if a reading or writing operation is currently in progress
				dSTinyDSS.getBusyFlag().then(function(Busy) {
					if (Busy) {
						console.log("Buttons were locked");
						$scope.ajax.ButtonsLocked = true;
					} else {
						console.log("Buttons were unlocked");
						$scope.ajax.ButtonsLocked = false;
					}
				}, function error() {
					console.log("Buttons were unlocked");
					$scope.ajax.ButtonsLocked = false;
				});
			};

			$scope.writeSelectedRegisters = function(dsid, toAll) {
				// write selected registers on the corresponding aribox or to all airboxes
				// The Boolean toAll indicates whether the values are to be written to all airboxes (true) or just to the one indicated by dsid.

				var title;
				var text;

				failed = function() {
					title = gettextCatalog.getString("Error")
					text = gettextCatalog.getString("The writing operation could not be completed!");
					$confirm({
						text: text,
						title: title,
						ok: 'OK',
						cancel: 'Cancel'
					});
					$scope.ajax.ButtonsLocked = false;
					$scope.ajax.TabsLocked = false;
				}

				dSTinyDSS.getBusyFlag().then(function(Busy) {
					if (Busy) {
						title = gettextCatalog.getString("Channel is busy")
						text = gettextCatalog.getString("Reading or writing operation in progress, please retry later")
						$confirm({
								text: text,
								title: title,
								ok: 'OK',
								cancel: 'Cancel'
							})
							.then(function() {
								return;
							});
						return;
					}



					var key = "k_" + dsid;

					if ($scope.tiny_conf_vars[key].type == 'ExHood') {
						dSTinyDSS.setExHoodVolumeLevels($scope.exhood_levels).then(function() {
							//alert("OK");
						});
					}
					var regs = []; // choose the selected (checked) registers
					for (var iv in $scope.tiny_conf_vars[key].variables) {
						var v = $scope.tiny_conf_vars[key].variables[iv];
						if (v.selected) {
							if (v.unit == '%') {
								if ($scope.tiny_conf_vars[key].type == 'Airbox') {
									regs.push([v.reg, parseInt(v.uservalue) + 100]); // this registers contains the checked variables' registers
								} else {
									regs.push([v.reg, parseInt(v.uservalue)]); // this registers contains the checked variables' registers
								}
							} else {
								regs.push([v.reg, v.uservalue]); // this registers contains the checked variables' registers
							}
						}
					}
					var estimated_time = 5 * regs.length;
					var eventTimeout = 8 * estimated_time * 1000; //ms

					if (regs.length > 0) {
						title = gettextCatalog.getString("Do you want to continue?");
						text = gettextCatalog.getString("This operation might take several minutes");
						$confirm({
								text: text,
								title: title,
								ok: 'OK',
								cancel: 'Cancel'
							})
							.then(function() {

								$scope.ajax.ButtonsLocked = true;
								$scope.ajax.TabsLocked = true;

								regs.push([3, "4"]); // save parameters to flash

								$scope.ajax.airbox_read_regs = true;
								//dSTinyDSS.readRegs(dsid,regs); // Read the registers

								if (toAll) {
									var whichFunction = dSTinyDSS.writeRegs_toAll;
								} else {
									var whichFunction = dSTinyDSS.writeRegs;
								}

								return dss.Event.unsubscribe('wesco_RegistersOpCompleted', $scope.ajax.OpCompletedSubscriptionId).finally(function() {
									return dss.Event.subscribe('wesco_RegistersOpCompleted', $scope.ajax.OpCompletedSubscriptionId)
										.then(function() {
											var type = $scope.tiny_conf_vars[key].type;
											return whichFunction(dsid, regs, type)
												.then(function(result) {
													return dss.Event.get($scope.ajax.OpCompletedSubscriptionId, eventTimeout)
														.then(function(result) {
															$scope.ajax.ButtonsLocked = false;
															$scope.ajax.TabsLocked = false;
															if (result.events[0] !== undefined) {
																if (result.events[0].properties !== undefined) {
																	var Dict = JSON.parse(result.events[0].properties.Dict);
																	var regs = JSON.parse(result.events[0].properties.Regs);
																	//console.log(regs);
																	title = gettextCatalog.getString("Success")
																	text = gettextCatalog.getString("{{N}} registers successfully transmitted!", {
																		N: regs.length - 1
																	});
																	$confirm({
																		text: text,
																		title: title,
																		ok: 'OK',
																		cancel: 'Cancel'
																	});
																} else {
																	failed();
																}
															} else {
																failed();
															}
															// $scope.ajax.airbox_read_regs = false;
														});
												});
										})
										.error(function() {
											failed();
										})
										.catch(function() {
											failed();
										});
								});
							});
					} // end if (regs.length > 0 ...
				}).catch(function(e) {
					console.log(e);
					failed();
				});


			}; // END of $scope.writeSelectedRegisters=function(dsid,toAll){


			$scope.importVarsFromFile = function(dsid, deviceType) {

				var selectModal = $modal.open({
					animation: true,
					templateUrl: 'importSelectModal.html',
					controller: 'importSelectModalCtrl',
					size: '',
				});

				selectModal.result.then(function(selection) {
					try {
						if (deviceType == 'Airbox') {
							var data = dSTinyDSS.parseVarFile(selection.data);
							$scope.importLog = importAirboxVars(data, dsid);
						} else if (deviceType == 'ExHood') {
							var data = dSTinyDSS.parseExHoodVarFile(selection.data);
							$scope.importLog = importExHoodVars(data, dsid);
						} else if (deviceType == 'Airbox-Volumes') {
							var data = dSTinyDSS.parseVarFile(selection.data);
							$scope.importLog = importAirboxVolumeVars(data);
						}
						var resultModal = $modal.open({
							animation: true,
							templateUrl: 'importResultModal.html',
							controller: 'importResultsModalCtrl', //shows OK
							size: '',
							resolve: {
								'importLog': function() {
									return $scope.importLog;
								},
							}
						});
					} catch (err) {
						alert(err);
					}
				});
			}; // END of $scope.importVarsFromFile = function(dsid){

			$scope.exportAirboxVarsToFile = function(dsid, deviceName) {
				try {
					var contents = "";
					var key = "k_" + dsid;
					contents += '#' + deviceName + '\t' + dsid + '\n#Reg\tValue\n';
					for (var iv in $scope.tiny_conf_vars[key].variables) {
						var v = $scope.tiny_conf_vars[key].variables[iv];
						var reg = v.reg;
						var value = v.uservalue;
						contents += reg + '\t' + value + '\n';
					}

					var blob = new Blob([contents], {
						type: "text/plain;charset=utf-8"
					});
					saveAs(blob, "Wesco_" + deviceName + "_" + dsid + ".txt");
				} catch (err) {
					alert(err);
				}
			}; // END of  $scope.exportAirboxVarsToFile = function() {

			$scope.exportAirboxVolumesToFile = function(dsid, deviceName) {
				try {
					var contents = "";
					//find any airbox key
					var key = undefined;
					for (var iv in $scope.tiny_conf_vars) {
						if ($scope.tiny_conf_vars[iv].type == 'Airbox') {
							key = 'k_' + $scope.tiny_conf_vars[iv].dsid;
							break;
						}
					}
					if (key !== undefined) {
						contents += '#Airboxes_VolumeRates' + '\n#Reg\tValue\n';
						for (var iv in $scope.tiny_conf_vars[key].variables) {
							var v = $scope.tiny_conf_vars[key].variables[iv];
							var reg = v.reg;
							var value = $scope.airbox_levels[v.level];
							contents += reg + '\t' + value + '\n';
						}

						var blob = new Blob([contents], {
							type: "text/plain;charset=utf-8"
						});
						saveAs(blob, "Wesco_AirboxesVolumeRates.txt");
					} else {
						alert("Configuration cannot be saved");
					}
				} catch (err) {
					alert(err);
				}
			}; // END of  $scope.exportAirboxVolumesToFile = function() {


			$scope.exportExHoodVarsToFile = function(dsid, deviceName) {
				try {
					var contents = "";
					var key = "k_" + dsid;
					contents += '#' + deviceName + '\t' + dsid + '\n#Reg\t%\tVolume (m3/h)\n';
					for (var iv in $scope.tiny_conf_vars[key].variables) {
						var v = $scope.tiny_conf_vars[key].variables[iv];
						var reg = v.reg;
						var value_procent = v.uservalue;
						if (v.cathegory == 'Fan') {
							var value_volume = $scope.exhood_levels[v.level];
						} else {
							var value_volume = "---";
						}
						contents += reg + '\t' + value_procent + '\t' + value_volume + '\n';
					}
					var blob = new Blob([contents], {
						type: "text/plain;charset=utf-8"
					});
					saveAs(blob, "Wesco_" + deviceName + "_" + dsid + ".txt");
				} catch (err) {
					alert(err);
				}
			}; // END of  $scope.exportExHoodVarsToFile = function() {


			function importAirboxVars(variables, dsid, setdefaults) {
				var importLog = {
					'changed': [],
					'unchanged': [],
					'undefined': [],
					'invalid': [],
				};


				var key = "k_" + dsid;
				for (var iv in variables) {
					var reg_index = "REG_" + variables[iv].reg; // retrieve variable register number
					var v = $scope.tiny_conf_vars[key].variables[reg_index];

					if (typeof v === 'undefined') {
						importLog['undefined'].push(variables[iv].reg);
					} else {
						var newvalue;
						if (typeof v.values == 'undefined') { // if it is a value with no options
							// regular continous value

							try {
								if (Number(variables[iv].value) == variables[iv].value) {
									newvalue = parseInt(variables[iv].value);
								} else {
									importLog['invalid'].push(variables[iv].reg);
									continue;
								}
							} catch (err) {
								importLog['invalid'].push(variables[iv].reg);
								continue;
							}

						} else {
							// values stored with string index
							newvalue = variables[iv].value;
						}
						if (newvalue > v.max || newvalue < v.min) {
							// verify range
							importLog['invalid'].push(variables[iv].reg);
						} else {
							if (newvalue != v.uservalue) {
								importLog['changed'].push(variables[iv].reg);
								v.uservalue = newvalue;
							} else {
								importLog['unchanged'].push(variables[iv].reg);
							}
							// Overwrite defaults
							if (setdefaults) {
								v.default = newvalue;
							}
						}
					}
				}
				return importLog;
			}; // END of function importAirboxVars

			function importAirboxVolumeVars(variables, setdefaults) {
				var importLog = {
					'changed': [],
					'unchanged': [],
					'undefined': [],
					'invalid': [],
				};


				//find any airbox key
				var key = undefined;
				for (var iv in $scope.tiny_conf_vars) {
					if ($scope.tiny_conf_vars[iv].type == 'Airbox') {
						key = 'k_' + $scope.tiny_conf_vars[iv].dsid;
						break;
					}
				}

				for (var iv in variables) {
					if (key === undefined) {
						importLog['undefined'].push(variables[iv].reg);
					} else {
						var reg_index = "REG_" + variables[iv].reg; // retrieve variable register number
						var v = $scope.tiny_conf_vars[key].variables[reg_index];

						if (typeof v === 'undefined') {
							importLog['undefined'].push(variables[iv].reg);
						} else {
							var newvalue = undefined;
							if (typeof v.values == 'undefined') { // if it is a value with no options
								// regular continous value
								try {
									if (Number(variables[iv].value) == variables[iv].value) {
										newvalue = parseInt(variables[iv].value);
									} else {
										importLog['invalid'].push(variables[iv].reg);
										continue;
									}
								} catch (err) {
									importLog['invalid'].push(variables[iv].reg);
									continue;
								}

							} else {
								// values stored with string index
								newvalue = variables[iv].value;
							}

							// assumes 150 m3/h is the maximum airbox volume rate
							if (newvalue !== undefined && (newvalue > MAXIMUM_AIRBOX_VOLUME || newvalue < 0)) {
								// verify range
								importLog['invalid'].push(variables[iv].reg);
							} else {
								if (newvalue != $scope.airbox_levels[v.level]) {
									importLog['changed'].push(variables[iv].reg);
									$scope.airbox_levels[v.level] = newvalue;
								} else {
									importLog['unchanged'].push(variables[iv].reg);
								}

							}
						}
					}
				}
				return importLog;
			}; // END of function importAirboxVolumeVars

			function importExHoodVars(variables, dsid, setdefaults) {
				var importLog = {
					'changed': [],
					'unchanged': [],
					'undefined': [],
					'invalid': [],
				};


				var key = "k_" + dsid;
				for (var iv in variables) {
					var reg_index = "REG_" + variables[iv].reg; // retrieve variable register number
					var v = $scope.tiny_conf_vars[key].variables[reg_index];

					if (typeof v === 'undefined') {
						importLog['undefined'].push(variables[iv].reg);
					} else {
						var newvalue = undefined;
						var volume_level = undefined;
						if (typeof v.values == 'undefined') { // if it is a value with no options
							// regular continous value
							try {
								if (Number(variables[iv].value) == variables[iv].value) {
									newvalue = parseInt(variables[iv].value);
								} else {
									importLog['invalid'].push(variables[iv].reg);
									continue;
								}
								if (v.cathegory === 'Fan') {
									if (Number(variables[iv].value2) == variables[iv].value2) {
										volume_level = parseInt(variables[iv].value2);
									} else {
										importLog['invalid'].push(variables[iv].reg);
										continue;
									}
								}
							} catch (err) {
								importLog['invalid'].push(variables[iv].reg);
								continue;
							}
						} else {
							// values stored with string index
							newvalue = variables[iv].value;
						}
						// If category is Fan, the volume level must have a valid
						if (v.cathegory == 'Fan') {
							if (Number(volume_level) != volume_level) {
								importLog['invalid'].push(variables[iv].reg);
							} else { // if the volume rate is a valid number. Assume 1000 m3/h is the maximum possible volume
								if ((volume_level < 0 || volume_level > MAXIMUM_EXHOOD_VOLUME) || (newvalue > v.max || newvalue < v.min)) {
									// verify range
									importLog['invalid'].push(variables[iv].reg);
								} else {
									if (newvalue != v.uservalue || volume_level != $scope.exhood_levels[v.level]) {
										importLog['changed'].push(variables[iv].reg);
										v.uservalue = newvalue;
										//update volume rate
										$scope.exhood_levels[v.level] = volume_level;
									} else {
										importLog['unchanged'].push(variables[iv].reg);
									}
									// Overwrite defaults
									if (setdefaults) {
										v.default = newvalue;
									}
								}
							}
						} else { // if not a fan register
							if (newvalue > v.max || newvalue < v.min) {
								// verify range
								importLog['invalid'].push(variables[iv].reg);
							} else {
								if (newvalue != v.uservalue) {
									importLog['changed'].push(variables[iv].reg);
									v.uservalue = newvalue;
								} else {
									importLog['unchanged'].push(variables[iv].reg);
								}
								// Overwrite defaults
								if (setdefaults) {
									v.default = newvalue;
								}
							}
						}
					}
				}
				return importLog;
			}; // END of function importExHoodVars
			$scope.loadAll = function() {
				var promises = [];
				promises.push($scope.checkifBusy());
				return $q.all(promises);
			};

			$scope.loadAll().then(function() {
				//console.log("Successfully loaded");
			}, function(err) {
				$scope.ajaxError = err;
				console.log(err);
			});



		}
	]); // end controller AirboxConf



// main application controller
airboxControllers.controller('OverviewCtrl', ['$q', '$scope', '$modal', '$timeout', 'dSTinyDSS', 'dss', 'gettextCatalog', '$location',
	function($q, $scope, $modal, $timeout, dSTinyDSS, dss, gettextCatalog, $location) {

		$scope.removeVentilationScheduleRow = function(idx) {
			$scope.ventilation_schedule.rules.splice(idx, 1);
		};

		$scope.addVentilationScheduleRow = function() {
			var rule = {};
			rule['day'] = $scope.ventilation_schedule.day.uservalue;
			var from = $scope.ventilation_schedule.from.uservalue;
			var until = $scope.ventilation_schedule.until.uservalue;

			try {
				if (parseInt(from) < parseInt(until)) {
					rule['from'] = from;
					rule['until'] = until;
					rule['Presence'] = $scope.ventilation_schedule.level.uservalue_presence;
					rule['Absence'] = $scope.ventilation_schedule.level.uservalue_absence;
					$scope.ventilation_schedule.rules.push(rule);
				}
			} catch (err) {
				console.log(err);
			}
		};

		function validate(v) {
			if (v == '0' || v == '1' || v == '2' || v == '3' || v == '4') {
				return true;
			} else {
				return false;
			}
		}

		function importVars(variables) {
			var importLog = {
				'changed': [],
				'new': [],
				'invalid': [],
			};


			for (var iv in variables) {
				var day = variables[iv].day;
				var from = variables[iv].from;
				var until = variables[iv].until;
				var presence = variables[iv].presence;
				var absence = variables[iv].absence;

				var counter = 0;
				for (r in $scope.ventilation_schedule.rules) {
					var rule = $scope.ventilation_schedule.rules[r];
					if ((rule.day == day && rule.from == from) && (rule.until == until)) {
						counter++;
						$scope.ventilation_schedule.rules.splice(r, 1); //delete the existing rule
						break;
					}
				}

				// validate and add rule
				try {

					day = parseInt(day);
					from = parseInt(from);
					until = parseInt(until);
					if (day <= 8) { //7 days pluse weekends and weekday
						if (from < until) {
							if (validate(presence) && validate(absence)) {
								var rule = {};
								rule['day'] = day;
								rule['from'] = from;
								rule['until'] = until;
								rule['Absence'] = absence;
								rule['Presence'] = presence;

								$scope.ventilation_schedule.rules.push(rule); // add new rule

								if (counter > 0 && counter <= $scope.ventilation_schedule.rules.length) {
									importLog['changed'].push(rule);
								} else {
									importLog['new'].push(rule);
								}
							}
						} else {
							console.log("Field from must be less than field until");
							importLog['invalid'].push(rule);
						}
					} else {
						console.log("Day is out of range");
						importLog['invalid'].push(rule);
					}
					//range and type
				} catch (err) {
					console.log(err);
					importLog['invalid'].push(0);
				}

			} // end for v in variables
			return importLog;
		}; // END of function importVars


		$scope.importVentilationScheduleFromFile = function(dsid) {

			var selectModal = $modal.open({
				animation: true,
				templateUrl: 'importVentilationScheduleModal.html',
				controller: 'importVentilationScheduleModalCtrl',
				size: '',
			});

			selectModal.result.then(function(selection) {
				try {
					$scope.importLog = importVars(selection.data);
					var resultModal = $modal.open({
						animation: true,
						templateUrl: 'importResultModal.html',
						controller: 'importResultsModalCtrl',
						size: '',
						resolve: {
							'importLog': function() {
								return $scope.importLog;
							},
						}
					});
				} catch (err) {
					alert(err);
				}
			});
		}; // END of $scope.importVarsFromFile = function(dsid){

		$scope.exportVentilationScheduleToFile = function() {
			try {
				var contents = "#Ventilation schedule configuration\n";
				contents += "# Each line is a rule\n";
				contents += "#Day\tFrom\tUntil\tPresence\tAbsence\n";

				for (r in $scope.ventilation_schedule.rules) {
					console.log(r);
					var rule = $scope.ventilation_schedule.rules[r];
					contents += rule.day + "\t" + rule.from + "\t" + rule.until + "\t" + rule.Presence + "\t" + rule.Absence + "\n";
				}

				var blob = new Blob([contents], {
					type: "text/plain;charset=utf-8"
				});
				saveAs(blob, "Wesco_time_plan.txt");
			} catch (err) {
				alert(err);
			}
		}; // END of  $scope.exportVentilationScheduleToFile = function() {



		$scope.saveVentilationSchedule = function() {
			var schedule = {};
			for (var r in $scope.ventilation_schedule.rules) {
				var rule = $scope.ventilation_schedule.rules[r];

				var day = rule.day;
				var values = {};
				values['from'] = rule.from;
				values['until'] = rule.until;
				values['Presence'] = rule.Presence;
				values['Absence'] = rule.Absence;
				if (!schedule[day]) {
					schedule[day] = {};
				}

				var rule_key = rule.from + '_' + rule.until;
				schedule[day][rule_key] = values;
			}
			//save rules and ventilation schedule
			return dSTinyDSS.configVentilationSchedule(schedule);

		};


		$scope.isActive = function(viewLocation) {
			return viewLocation === $location.path();
		};

		$scope.initializeVentilationSchedule = function() {
			// Airbox schedule configuration
			var ventilation_schedule = {
				'rules': [],
				'day': {
					'uservalue': '7',
					'values': {
						'0': gettextCatalog.getString("Sunday"),
						'1': gettextCatalog.getString("Monday"),
						'2': gettextCatalog.getString("Tuesday"),
						'3': gettextCatalog.getString("Wednesday"),
						'4': gettextCatalog.getString("Thursday"),
						'5': gettextCatalog.getString("Friday"),
						'6': gettextCatalog.getString("Saturday"),
						'7': gettextCatalog.getString("Mon-Fri"),
						'8': gettextCatalog.getString("Weekend"),
					}
				},
				'hour': {
					'uservalue': '0',
					'values': ['0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20', '21', '22', '23']
				},
				'level': {
					'uservalue_presence': '0',
					'uservalue_absence': '0',
					'values': {
						'0': gettextCatalog.getString("Off"),
						'1': gettextCatalog.getString("Low"),
						'2': gettextCatalog.getString("Medium"),
						'3': gettextCatalog.getString("Nominal"),
						'4': gettextCatalog.getString("Full"),
					}
				}
			};
			//

			$scope.ventilation_schedule = ventilation_schedule;
			dSTinyDSS.getVentilationSchedule().then(function(rules) {
				$scope.ventilation_schedule.rules = rules;
			});

		};


		// Global ajax error message to display
		$scope.ajaxError = false;


		// Ajax request states to show/hide elements depending on if content is loaded
		$scope.ajax = {
			'levelReady': false,
			'settingLevel': false,
			'settingConfig': false,
			'exhaustFlapCandidatesReady': false,
			'pushTargetOptionsReady': false,
			'configReady': false,
			'errorlogReady': false,
			'deviceListReady': false,
			'AirboxListReady': false,
			'ExhoodListReady': false,
			'WCFansListReady': false,
		};

		// Variables for pagination of logfiles
		$scope.logTable = {
			'maxSize': 10,
			'currentPage': 1,
		};

		// Translations of error texts
		$scope.airbox_error_texts = {
			"NOERROR": "OK",
			"POWERERR": "Power Fail",
			"SENSORERR": "Sensor Error",
			"FANERR": "Fan Error",
			"FLAPERR": "Flap Error",
			"FROSTERR": "Freezing Error",
			"LOCKED": "Operation temporally locked",
			"FAN_MISSING": "1 Fan is missing",
			"NEWAIRBOX": "New Airbox discovered",
			"NEWEXHOOD": "New ExHood discovered",
			"NEWWCFAN": "New WCFan discovered",
			"WCFANLOST": "WCFan is not longer available",
			"EXHOODLOST": "Exhaust hood is not longer available",
			"AIRBOXLOST": "Airbox is not longer available",
			"TMOUTERR": "Timeout",
		};

		$scope.refreshCurrentLevel = function() {
			$scope.ajax.levelReady = false;
			return dSTinyDSS.getCurrentLevel().then(function(currentLevel) {
				$scope.currentLevel = currentLevel;
				$scope.ajax.levelReady = true;
			});
		};


		$scope.refreshExhoodCurrentValue_fan = function() {
			return dSTinyDSS.getExhoodCurrentValue_fan().then(function(currentLevel) {
				$scope.ExhoodCurrentValue_fan = currentLevel;
			});
		};

		$scope.refreshExhoodCurrentValue_flap = function() {
			return dSTinyDSS.getExhoodCurrentValue_flap().then(function(currentLevel) {
				$scope.ExhoodCurrentValue_flap = currentLevel;
			});
		};

		$scope.refreshConfig = function() {
			$scope.ajax.configReady = false;
			return dSTinyDSS.getConfig().then(function(config) {
				$scope.config = config;

				$scope.selectedPushTargets = {};
				for (var i = 0; i < $scope.config.PushTargets.length; i++) {
					$scope.selectedPushTargets[$scope.config.PushTargets[i]] = true;
				}
				$scope.ajax.configReady = true;
			});
		};

		$scope.refreshErrorLog = function() {
			$scope.ajax.errorlogReady = false;
			return dSTinyDSS.getErrorLog().then(function(errorlog) {
				$scope.errorLog = [];
				for (var i = 0; i < errorlog.length; i++) {
					$scope.errorLog.push({
						'datetime': errorlog[i][0],
						'type': errorlog[i][1],
						'dSID': errorlog[i][2],
						'name': $scope.Airboxes.nameById[errorlog[i][2]] ? $scope.Airboxes.nameById[errorlog[i][2]] : errorlog[i][2],
						'errorcode': errorlog[i][3],
					});
				}

				$scope.ajax.errorlogReady = true;
			});
		};

		$scope.refreshExhaustFlapCandidates = function() {
			$scope.ajax.exhaustFlapCandidatesReady = false;
			return dSTinyDSS.getExhaustFlapCandidates().then(function(devlist) {
				$scope.exhaustFlapCandidates = devlist;
				$scope.ajax.exhaustFlapCandidatesReady = true;
			})
		};

		$scope.refreshPushTargetOptions = function() {
			$scope.ajax.pushTargetOptionsReady = false;
			return dSTinyDSS.refreshPushTargetOptions().then(function() {
				return $timeout(function() {
					return dSTinyDSS.getPushTargetOptions().then(function(targets) {
						$scope.pushTargetOptions = targets;
						$scope.ajax.pushTargetOptionsReady = true;
					})
				}, 3000);
			});
		};

		$scope.clearAllAirboxErrors = function() {
			var answer = confirm("Achtung: Nach der Fehlerbehebung müssen die betroffenen Airboxen neugestartet werden. Wenn die Störung weiterhin besteht ist dies nach dem Entfernen der Störungsmeldungen nicht sofort ersichtlich. Bitte überprüfen Sie in 30 Minuten ob der Fehler nun behoben ist.")
			if (!answer) {
				$scope.refreshConfig();
				return;
			}

			$scope.ajax.deviceListReady = false;
			return dSTinyDSS.clearAllAirboxErrors().then(function() {
				return $timeout(function() {
					return $scope.refreshAirboxes();
				}, 3000);
			});
		};

		$scope.clearAirboxError = function(dSUID) {

			var answer = confirm(gettextCatalog.getString("Before proceeding, please make sure that the error was correctly removed and the Airbox restarted. Refer to the user manual"));
			if (!answer) {
				$scope.refreshConfig();
				return;
			}

			return dSTinyDSS.clearAirboxError(dSUID).then(function() {
				return $timeout(function() {
					return $scope.refreshAirboxes();
				}, 3000);
			});
		};


		$scope.updateBaseVentilation = function() {
			return dSTinyDSS.setBaseVentilation($scope.config.BaseVentilationEnabled, $scope.config.BaseVentilationLevel);
		};

		$scope.updateConfig = function() {
			// var answer = confirm("Drücken Sie OK um die Konfiguration zu speichern.")
			// if (!answer) {
			// 	  $scope.refreshConfig();
			// 	  return;
			// }

			var promises = [];
			$scope.ajax.settingConfig = false;

			// Create list of selected UUIDs
			$scope.config.PushTargets = [];
			for (var UUID in $scope.selectedPushTargets) {
				if ($scope.selectedPushTargets.hasOwnProperty(UUID)) {
					if ($scope.selectedPushTargets[UUID]) {
						$scope.config.PushTargets.push(UUID);
					}
				}
			}

			promises.push(dSTinyDSS.setPushTargets($scope.config.PushTargets));
			//	promises.push(dSTinyDSS.setExhaustFlap($scope.config.ExhaustFlap));
			promises.push(dSTinyDSS.setEmailAddress($scope.config.EmailAddress));

			//@Diego
			// promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,72)); // Scene 72 -> leave home
			// promises.push(dss.Scene.disableScene($scope.config.ExhaustFlap,69)); // Scene 69 -> sleeping
			promises.push(dSTinyDSS.storeConfigurableParameters());


			return $q.all(promises).then(function() {
				$scope.ajax.settingConfig = true;
			});
		};



		//@Diego
		$scope.restartAirboxes = function() {
			var promises = [];
			promises.push(dSTinyDSS.restartAirboxes());
		};

		/*********************************************************************/
		// $scope.refreshDevices = function() {
		//     var promises = [];
		//     if(typeof $scope['Airboxes'] == 'undefined') {
		// 	  $scope['Airboxes'] = {
		// 	      'nameById': {},
		// 	  };
		//     }

		//     if(typeof $scope['ExHoods'] == 'undefined') {
		// 	  $scope['ExHoods'] = {
		// 	      'nameById': {},
		// 	  };
		//     }


		//     if(typeof $scope['WCFans'] == 'undefined') {
		// 	  $scope['WCFans'] = {
		// 	      'nameById': {},
		// 	  };
		//     }


		//     $scope.ajax.deviceListReady = false;


		//     dSTinyDSS.rediscoverDevices();
		//     // Wait 5 seconds for background script to update property tree
		//     return $timeout(function() {
		// 	  promises.push(dSTinyDSS.getAirboxConfig().then(function(config) {
		// 	      for (var attrname in config) {
		// 		  $scope['Airboxes'][attrname] = config[attrname];
		// 	      }
		// 	  }));

		// 	  // get airboxes
		// 	  promises.push(dSTinyDSS.getAirboxes().then(function(airboxes) {
		// 	      $scope.Airboxes.Devices = airboxes;
		// 	      $scope.Airboxes.Error = false;
		// 	      // Create index to resolve name by dSUID
		// 	      for (var i = 0; i < $scope.Airboxes.Devices.length; i++) {
		// 		  if($scope.Airboxes.Devices[i].errorcode!='NOERROR') {
		// 		      $scope.Airboxes.Error = true;
		// 		  }
		// 		  $scope.Airboxes.nameById[$scope.Airboxes.Devices[i].dSID] = $scope.Airboxes.Devices[i].Description;
		// 	      };
		// 	  }));

		// 	  // get exhoods
		// 	  promises.push(dSTinyDSS.getExHoods().then(function(exhoods) {
		// 	      $scope.ExHoods.Devices = exhoods;
		// 	      $scope.ExHoods.Error = false;
		// 	      // Create index to resolve name by dSUID
		// 	      for (var i = 0; i < $scope.ExHoods.Devices.length; i++) {
		// 		  if($scope.ExHoods.Devices[i].errorcode!='NOERROR') {
		// 		      $scope.ExHoods.Error = true;
		// 		  }
		// 		  $scope.ExHoods.nameById[$scope.ExHoods.Devices[i].dSID] = $scope.ExHoods.Devices[i].Description;
		// 	      };
		// 	  }));


		// 	  // get WCFans
		// 	  promises.push(dSTinyDSS.getWCFans().then(function(wcfans) {
		// 	      $scope.WCFans.Devices = wcfans;
		// 	      // Create index to resolve name by dSUID
		// 	      for (var i = 0; i < $scope.WCFans.Devices.length; i++) {
		// 		  $scope.WCFans.nameById[$scope.WCFans.Devices[i].dSID] = $scope.WCFans.Devices[i].Description;
		// 	      };
		// 	  }));



		// 	  return $q.all(promises).then(function() {
		// 	      $scope.ajax.deviceListReady = true;
		// 	  });
		//     },3000);
		// };

		/*********************************************************************/
		$scope.refreshWCFans = function() {
			var promises = [];

			if (typeof $scope['WCFans'] == 'undefined') {
				$scope['WCFans'] = {
					'nameById': {},
				};
			}


			$scope.ajax.WCFansListReady = false;


			dSTinyDSS.rediscoverWCFans();
			// Wait 5 seconds for background script to update property tree
			return $timeout(function() {

				// get WCFans
				promises.push(dSTinyDSS.getWCFans().then(function(wcfans) {
					$scope.WCFans.Devices = wcfans;
					// Create index to resolve name by dSUID
					for (var i = 0; i < $scope.WCFans.Devices.length; i++) {
						$scope.WCFans.nameById[$scope.WCFans.Devices[i].dSID] = $scope.WCFans.Devices[i].Description;
					};
				}));



				return $q.all(promises).then(function() {
					$scope.ajax.WCFansListReady = true;
				});
			}, 3000);
		};

		/*********************************************************************/
		$scope.refreshExHoods = function() {
			var promises = [];

			if (typeof $scope['ExHoods'] == 'undefined') {
				$scope['ExHoods'] = {
					'nameById': {},
				};
			}


			$scope.ajax.ExhoodListReady = false;


			dSTinyDSS.rediscoverExhoods();
			// Wait 5 seconds for background script to update property tree
			return $timeout(function() {

				// get exhoods
				promises.push(dSTinyDSS.getExHoods().then(function(exhoods) {
					$scope.ExHoods.Devices = [];
					for (var i in exhoods) {
						if (exhoods[i].Type == 'Fan_Flap') { // select only exhood device with dsxid=1
							$scope.ExHoods.Devices.push(exhoods[i]);
						}
					}
					$scope.ExHoods.Error = false;

					// Create index to resolve name by dSUID
					for (var i = 0; i < $scope.ExHoods.Devices.length; i++) {
						if ($scope.ExHoods.Devices[i].errorcode != 'NOERROR') {
							$scope.ExHoods.Error = true;
						}

						$scope.ExHoods.nameById[$scope.ExHoods.Devices[i].dSID] = $scope.ExHoods.Devices[i].Description;
					};
				}));



				return $q.all(promises).then(function() {
					$scope.ajax.ExhoodListReady = true;
				});
			}, 3000);
		};

		/******************************************************************************/
		$scope.refreshAirboxes = function() {
			var promises = [];
			if (typeof $scope['Airboxes'] == 'undefined') {
				$scope['Airboxes'] = {
					'nameById': {},
				};
			}

			$scope.ajax.AirboxListReady = false;


			dSTinyDSS.rediscoverAirboxes();
			// Wait 5 seconds for background script to update property tree
			return $timeout(function() {
				promises.push(dSTinyDSS.getAirboxConfig().then(function(config) {
					for (var attrname in config) {
						$scope['Airboxes'][attrname] = config[attrname];
					}
				}));

				// get airboxes
				promises.push(dSTinyDSS.getAirboxes().then(function(airboxes) {
					$scope.Airboxes.Devices = airboxes;
					$scope.Airboxes.Error = false;
					// Create index to resolve name by dSUID
					for (var i = 0; i < $scope.Airboxes.Devices.length; i++) {
						if ($scope.Airboxes.Devices[i].errorcode != 'NOERROR') {
							$scope.Airboxes.Error = true;
						}
						$scope.Airboxes.nameById[$scope.Airboxes.Devices[i].dSID] = $scope.Airboxes.Devices[i].Description;
					};
				}));



				return $q.all(promises).then(function() {
					$scope.ajax.AirboxListReady = true;
				});
			}, 3000);
		};



		$scope.setLevel = function(level) {
			$scope.ajax.settingLevel = true;
			dSTinyDSS.setLevel(level).then(function() {
				$scope.ajax.settingLevel = false;
			});
		};

		$scope.initDSS = function() {
			// get locale, if german set ui to german otherwise use english
			return dss.Property.getString("/system/language/locale").then(function(locale) {
				gettextCatalog.setCurrentLanguage(locale.substr(0, 2));
				//if(locale.substr(0,2)=="de") {
				//  }
			});
		};

		$scope.getCookingMode = function() {
			dSTinyDSS.getPresenceMode().then(function(Mode) {
				if (Mode == "Cooking") {
					$scope.ajax.SceneButtonsLocked = true;
				} else {
					$scope.ajax.SceneButtonsLocked = false;
				}
			});
		};

		$scope.getBigMode = function() {
			dSTinyDSS.getBigMode().then(function(Mode) {
				if (Mode == "Alarm") {
					$scope.ajax.SceneButtonsLocked = true;
				} else {
					$scope.ajax.SceneButtonsLocked = false;
				}
			});
		};

		$scope.loadAll = function() {
			var promises = [];
			promises.push($scope.initDSS());
			promises.push($scope.refreshExHoods());
			promises.push($scope.refreshConfig());
			promises.push($scope.refreshAirboxes());
			promises.push($scope.refreshWCFans());
			promises.push($scope.refreshPushTargetOptions());
			promises.push($scope.initializeVentilationSchedule());
			promises.push($scope.getCookingMode());
			promises.push($scope.getBigMode());

			return $q.all(promises);
		};
		(function pollLevel() {
			$scope.refreshCurrentLevel().then(
				$scope.refreshExhoodCurrentValue_flap().then(
					$scope.refreshExhoodCurrentValue_fan().then(function() {
						$scope.getCookingMode();
						$scope.getBigMode();
						$timeout(pollLevel, 3000);
					})));
		})();
		$scope.loadAll().then(function() {
				return $scope.refreshErrorLog();
			},
			function(err) {
				$scope.ajaxError = err;
				console.log(err);
			});

	}
]);


airboxControllers.controller('importSelectModalCtrl', ['$scope', '$modalInstance', 'dSTinyDSS',
	function($scope, $modalInstance, dSTinyDSS) {
		$scope.ok = function() {
			var file = document.getElementById('selectedFile').files[0];
			if (file.type.match(/text.*/)) {
				var reader = new FileReader();
				reader.onload = function(e) {
					$modalInstance.close({
						'source': 'file',
						'data': reader.result,
					});
				}
				reader.readAsText(file);
			} else {
				throw "ERROR: Could not read file. Only plain text files with tab-separated values can be imported.";
			}

		};
		$scope.cancel = function() {
			$modalInstance.dismiss('cancel');
		};
	}
]);


airboxControllers.controller('importVentilationScheduleModalCtrl', ['$scope', '$modalInstance', 'dSTinyDSS',
	function($scope, $modalInstance, dSTinyDSS) {
		$scope.ok = function() {
			var file = document.getElementById('selectedFile').files[0];
			if (file.type.match(/text.*/)) {
				var reader = new FileReader();
				reader.onload = function(e) {
					$modalInstance.close({
						'source': 'file',
						'data': dSTinyDSS.parseVentilationScheduleFile(reader.result),
					});
				}
				reader.readAsText(file);
			} else {
				throw "ERROR: Could not read file. Only plain text files with tab-separated values can be imported.";
			}

		};
		$scope.cancel = function() {
			$modalInstance.dismiss('cancel');
		};
	}
]);


airboxControllers.controller('importResultsModalCtrl',
	function($scope, $modalInstance, importLog) {
		$scope.importLog = importLog;
		$scope.ok = function() {
			$modalInstance.close('ok');
		}
	});